/*
 * Copyright 2010-2011 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  http://aws.amazon.com/apache2.0
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package edu.columbia.cs.irt.sipcloud;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import com.amazonaws.AmazonClientException;
import com.amazonaws.AmazonServiceException;
import com.amazonaws.auth.AWSCredentials;
import com.amazonaws.auth.PropertiesCredentials;
import com.amazonaws.services.cloudwatch.AmazonCloudWatchClient;
import com.amazonaws.services.cloudwatch.model.Datapoint;
import com.amazonaws.services.cloudwatch.model.Dimension;
import com.amazonaws.services.cloudwatch.model.GetMetricStatisticsRequest;
import com.amazonaws.services.cloudwatch.model.GetMetricStatisticsResult;
import com.amazonaws.services.ec2.AmazonEC2;
import com.amazonaws.services.ec2.AmazonEC2Client;
import com.amazonaws.services.ec2.model.AuthorizeSecurityGroupIngressRequest;
import com.amazonaws.services.ec2.model.CreateTagsRequest;
import com.amazonaws.services.ec2.model.DescribeInstancesRequest;
import com.amazonaws.services.ec2.model.DescribeInstancesResult;
import com.amazonaws.services.ec2.model.Instance;
import com.amazonaws.services.ec2.model.InstanceState;
import com.amazonaws.services.ec2.model.IpPermission;
import com.amazonaws.services.ec2.model.Reservation;
import com.amazonaws.services.ec2.model.RunInstancesRequest;
import com.amazonaws.services.ec2.model.RunInstancesResult;
import com.amazonaws.services.ec2.model.Tag;
import com.amazonaws.services.ec2.model.TerminateInstancesRequest;
import com.amazonaws.services.ec2.model.TerminateInstancesResult;


/**
 * Welcome to your new AWS Java SDK based project!
 *
 * This class is meant as a starting point for your console-based application that
 * makes one or more calls to the AWS services supported by the Java SDK, such as EC2,
 * SimpleDB, and S3.
 *
 */
public class AwsCenter {
	int privateID; /* for tracking VM creation */

    /*
     * Important: Be sure to fill in your AWS access credentials in the
     *            AwsCredentials.properties file before you try to run this
     *            sample.
     * http://aws.amazon.com/security-credentials
     */
    static AmazonEC2    ec2east;
    static AmazonEC2	ec2west;

    /**
     * The only information needed to create a client are security credentials
     * consisting of the AWS Access Key ID and Secret Access Key. All other
     * configuration, such as the service endpoints, are performed
     * automatically. Client parameters, such as proxies, can be specified in an
     * optional ClientConfiguration object when constructing a client.
     *
     * @see com.amazonaws.auth.BasicAWSCredentials
     * @see com.amazonaws.auth.PropertiesCredentials
     * @see com.amazonaws.ClientConfiguration
     */
    public void init() throws Exception {
   		AWSCredentials credentials = new PropertiesCredentials(
        		AwsCenter.class.getResourceAsStream("AwsCredentials.properties"));

       	ec2east = new AmazonEC2Client(credentials);
       	ec2east.setEndpoint(Constants.ec2UsEast);
       	
       	ec2west = new AmazonEC2Client(credentials);
       	ec2west.setEndpoint(Constants.ec2UsWest);
       	
       	privateID = 0;
    }
    
    public void fillVmList(List<VmElement> vmList) {
    	fillVmList(ec2east,vmList);
    	fillVmList(ec2west,vmList);
    }

    private void fillVmList(AmazonEC2 ec2, List<VmElement> vmList) {
        try {
            DescribeInstancesResult describeInstancesRequest = ec2.describeInstances();
            List<Reservation> reservations = describeInstancesRequest.getReservations();
            Set<Instance> instances = new HashSet<Instance>();

            for (Reservation reservation : reservations) {
                instances.addAll(reservation.getInstances());
            }
                   
            System.out.println("There are " + instances.size() + " Amazon EC2 instance(s). May not be running.");
            
            for (Instance instance: instances) {
            	// Don't add non-running instances
            	InstanceState is = instance.getState();
            	if (is.getName().compareTo("running") != 0) {
            		continue;
            	}
            	
            	VmElement e = new VmElement();
            	e.type = Constants.TYPE_NA;
            	List<Tag> tags = instance.getTags();
            	for (Tag tag: tags) {
            		if (tag.getKey().compareTo(Constants.TAG_TYPE) == 0) {
            			if ( tag.getValue().compareTo(Constants.TYPE_CONF) == 0 ||
            				 tag.getValue().compareTo(Constants.TYPE_CASS) == 0 ||
            				 tag.getValue().compareTo(Constants.TYPE_SER) == 0 ||
            				 tag.getValue().compareTo(Constants.TYPE_RELAY) == 0 ||
            				 tag.getValue().compareTo(Constants.TYPE_LB) == 0 ) {
            				e.type = tag.getValue();
            				e.instanceID = instance.getInstanceId();
                        	e.privAddr = instance.getPrivateIpAddress();
                        	e.pubAddr = instance.getPublicIpAddress();
                        	e.region = (ec2.equals(ec2east))?Constants.ec2UsEast:Constants.ec2UsWest;
                        	e.processRunning = true;
                        	
                        	vmList.add(e);
            			}
            		} 
            		if (tag.getKey().compareTo(Constants.TAG_SEED) == 0) {
            			if ( tag.getValue().compareTo("true") == 0) {
            				e.isSeed = true;
            				LoadScalingManager.cassandraSeed = e;
            			}
            		}
            	}
            	if (e.type.compareTo(Constants.TYPE_NA) == 0) {
            		System.err.println("Type is unknown for instance ID: " + e.instanceID);
            	}
            	
            	
            }
            
        } catch (AmazonServiceException ase) {
                System.out.println("Caught Exception: " + ase.getMessage());
                System.out.println("Reponse Status Code: " + ase.getStatusCode());
                System.out.println("Error Code: " + ase.getErrorCode());
                System.out.println("Request ID: " + ase.getRequestId());
        } catch (AmazonClientException ace) {
            /*
             * AmazonClientExceptions represent an error that occurred inside
             * the client on the local host, either while trying to send the
             * request to AWS or interpret the response. For example, if no
             * network connection is available, the client won't be able to
             * connect to AWS to execute a request and will throw an
             * AmazonClientException.
             */
            System.out.println("Error Message: " + ace.getMessage());
        } 
    }
    
    public VmElement addInstance(String region, String type) {
    	
    	AmazonEC2 ec2 = (region.compareTo(Constants.ec2UsEast)==0)?ec2east:ec2west;
    	String imageId = (region.compareTo(Constants.ec2UsEast)==0)?Constants.imageId:Constants.imageId_uswest2;
    	String confImageId = (region.compareTo(Constants.ec2UsEast)==0)?Constants.confImageId:Constants.confImageId_uswest2;
    	
    	VmElement e = null;	      // will return this.
    	
        int minInstanceCount = 1; // create 1 instance
        int maxInstanceCount = 1;
        RunInstancesRequest riRequest = new RunInstancesRequest(imageId, minInstanceCount, maxInstanceCount);
        
        //set instance type.
        riRequest.setInstanceType(Constants.instanceType.toString());
        
        
        // different image and instance type if this is a conference
        if( type.compareTo(Constants.TYPE_CONF) == 0) {
        	riRequest = new RunInstancesRequest(confImageId, minInstanceCount, maxInstanceCount);
        	riRequest.setInstanceType(Constants.confInstanceType.toString());
        }
        
        //set key pair.
        riRequest.setKeyName(Constants.keyname);
        
        //set security group.
        List<String> sGroup = new LinkedList<String>();
        sGroup.add(Constants.securityGroup);
        riRequest.setSecurityGroups(sGroup);
        
        // set monitoring
        riRequest.setMonitoring(false);		// detailed monitoring
        
        //create an instance
        try {
        	Date startDate = new Date();
	        RunInstancesResult result = ec2.runInstances(riRequest);
	        long startTime = startDate.getTime();
	        List<Instance> resultInstance = result.getReservation().getInstances();
	        Instance ins = resultInstance.get(0);
	        
	        // instance ID returns right away. Need this for later.
	        String instanceID = ins.getInstanceId();
	        
	        // To track VM creation process 
	        e = new VmElement();
	        e.privateID = this.privateID;
	        e.region = region;
	        this.privateID = this.privateID + 1;
	        LoadScalingManager.vmList.add(e);
	  
	        // wait for the VM to boot
	        Thread.sleep(Constants.vmBootTime);
	        
	        // wait for the instance to become ready
	        if ( !waitForInstanceReady(ec2,instanceID) ) {
	        	System.err.println("Error in waitForInstanceReady!");
	        	return null;
	        }
	        
	        // log time
	        Date endDate = new Date();
			long endTime = endDate.getTime();
			LoadScalingManager.logger.info(type + ": VM startup took: " + (endTime - startTime) + " milliseconds.");
	        
	        // ask VM about its information (public IP, private IP, etc)
	        DescribeInstancesRequest dir = new DescribeInstancesRequest();
			List<String> instances = new ArrayList<String>();
			instances.add(instanceID);		// set to the new VM instance we just launched.
			dir.setInstanceIds(instances);	
			
			// Let's ask for information about this instance.
			DescribeInstancesResult diResult = ec2.describeInstances(dir);
			List<Reservation> reservations = diResult.getReservations();
            List<Instance> instances2 = new ArrayList<Instance>();

            for (Reservation reservation : reservations) {
                instances2.addAll(reservation.getInstances());
            }
			
			// there really should be only one instance in here.
            Instance instance1 = instances2.get(0);
			
    		 //get instanceId from the result
        	System.out.println("New instance has been created:");
        	System.out.println("\t instanceId="+instance1.getInstanceId());
        	System.out.println("\t public IP="+instance1.getPublicIpAddress());
        	System.out.println("\t private IP="+instance1.getPrivateIpAddress());
        	
        	//e = new VmElement();
        	for (VmElement te: LoadScalingManager.vmList) {
        		if (te.privateID == e.privateID) {
        			LoadScalingManager.vmList.remove(te);
        			break;
        		}
        	}
        	e.instanceID = instance1.getInstanceId();
        	e.pubAddr = instance1.getPublicIpAddress();
        	e.privAddr = instance1.getPrivateIpAddress();
        	e.region = region;
        	e.type = type;
            
	        	
        } catch (AmazonServiceException ase) {
            System.out.println("Caught Exception: " + ase.getMessage());
            System.out.println("Reponse Status Code: " + ase.getStatusCode());
            System.out.println("Error Code: " + ase.getErrorCode());
            System.out.println("Request ID: " + ase.getRequestId());
	    } catch (AmazonClientException ace) {
	        /*
	         * AmazonClientExceptions represent an error that occurred inside
	         * the client on the local host, either while trying to send the
	         * request to AWS or interpret the response. For example, if no
	         * network connection is available, the client won't be able to
	         * connect to AWS to execute a request and will throw an
	         * AmazonClientException.
	         */
	        System.out.println("Error Message: " + ace.getMessage());
	    } catch (InterruptedException e1) {
			e1.printStackTrace();
		}
	    
	    return e;
    }
    
    // waitForInstanceReady 
    public boolean waitForInstanceReady(AmazonEC2 ec2, String instanceID) {
    	boolean ready = false;
    	
    	while (!ready) {
    		// ask VM about its information (public IP, private IP, etc)
	        DescribeInstancesRequest dir = new DescribeInstancesRequest();
			List<String> instances = new ArrayList<String>();
			instances.add(instanceID);		// set to the new VM instance we just launched.
			dir.setInstanceIds(instances);	
			
			// Let's ask for information about this instance.
			DescribeInstancesResult diResult = ec2.describeInstances(dir);
			List<Reservation> reservations = diResult.getReservations();
            List<Instance> instances2 = new ArrayList<Instance>();

            for (Reservation reservation : reservations) {
                instances2.addAll(reservation.getInstances());
            }
			
			// there really should be only one instance in here.
            Instance instance1 = instances2.get(0);
			
    		 //get instanceId from the result
            String pubAddr = instance1.getPublicIpAddress();
            if ( pubAddr != null) {
            	System.out.println("Public IP Address assigned: " + pubAddr);
            	ready = true;
            }
			
			try {
				if (ready) 
					Thread.sleep(70000);		// give time for sshd to run
				else
					Thread.sleep(3000);
			} catch (InterruptedException e) {
				e.printStackTrace();
				return false;
			}
    	}
		return ready;
    }
    
    public void removeInstance(VmElement e) {
    	if (e.region.compareTo(Constants.ec2UsEast) == 0) 
    		removeInstance(ec2east,e);
    	else
    		removeInstance(ec2west,e);
    }
    
    private void removeInstance(AmazonEC2 ec2, VmElement e) {
    	
        List<String> instances = new LinkedList<String>();
        instances.add(e.instanceID);
        TerminateInstancesRequest tiRequest = new TerminateInstancesRequest(instances);
             
        try {
        	TerminateInstancesResult result = ec2.terminateInstances(tiRequest);
	        	
        } catch (AmazonServiceException ase) {
            System.out.println("Caught Exception: " + ase.getMessage());
            System.out.println("Reponse Status Code: " + ase.getStatusCode());
            System.out.println("Error Code: " + ase.getErrorCode());
            System.out.println("Request ID: " + ase.getRequestId());
	    } catch (AmazonClientException ace) {
	        /*
	         * AmazonClientExceptions represent an error that occurred inside
	         * the client on the local host, either while trying to send the
	         * request to AWS or interpret the response. For example, if no
	         * network connection is available, the client won't be able to
	         * connect to AWS to execute a request and will throw an
	         * AmazonClientException.
	         */
	        System.out.println("Error Message: " + ace.getMessage());
	    } 
	    
	    return;
    }
    
    public void tagInstance(VmElement e) {
    	if (e.region.compareTo(Constants.ec2UsEast) == 0) 
    		tagInstance(ec2east,e);
    	else
    		tagInstance(ec2west,e);
    }
    
    private void tagInstance(AmazonEC2 ec2, VmElement e) {
    	
    	// Resources to tag: by instanceID.
    	List<String> instanceIDs = new ArrayList<String>();
    	instanceIDs.add(e.instanceID);
    	
    	// Tags themselves.
    	List<Tag> tags = new ArrayList<Tag>();
    	Tag tType = new Tag(Constants.TAG_TYPE, e.type);
    	Tag tSeed = null;
    	if (LoadScalingManager.cassandraSeed == e) {
    		tSeed = new Tag(Constants.TAG_SEED, "true");
    	} else {
    		tSeed = new Tag(Constants.TAG_SEED, "false");
    	}
    	tags.add(tType);
    	tags.add(tSeed);
    	
    	CreateTagsRequest ctr = new CreateTagsRequest(instanceIDs, tags);
    	ec2.createTags(ctr);

    }
    
    public void updateSecurityGroup(VmElement cass) {
    	List<IpPermission> rules = new ArrayList<IpPermission>();
		Collection<String> cidrList = new ArrayList<String>();
		cidrList.add(cass.pubAddr + "/32");
			
		IpPermission rule = new IpPermission();
		rule.setIpRanges(cidrList);
		rule.setIpProtocol("tcp");
		rule.setToPort(7000);
		rule.setFromPort(7000);
			
		rules.add(rule);

		// if element is in us-east, add rule to us-west to allow it to connect to us-west
		AmazonEC2 ec2 = (cass.region.compareTo(Constants.ec2UsEast)==0)?ec2west:ec2east;

    	AuthorizeSecurityGroupIngressRequest ingressReq = new AuthorizeSecurityGroupIngressRequest();
    	ingressReq.setGroupName(Constants.securityGroup);
    	ingressReq.setIpPermissions(rules);
    	ec2.authorizeSecurityGroupIngress(ingressReq);
    }

    /**
     * Get VM Statistic with the given instance ID
     * 
     * @param instanceID Amazon EC2 Instance ID
     * @return Data String, which includes CPU, Network In, Network Out
     */
	public ArrayList<List<Datapoint>> getVMStatistic(String instanceID) {
		ArrayList <List<Datapoint>> resultList = new ArrayList<List<Datapoint>>();
		AWSCredentials credentials = null;
		try {
			credentials = new PropertiesCredentials(
					AwsCenter.class.getResourceAsStream("AwsCredentials.properties"));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		AmazonCloudWatchClient cli = new AmazonCloudWatchClient(credentials);
		GetMetricStatisticsRequest req = new GetMetricStatisticsRequest();
		
		ArrayList<Dimension> dimensions = new ArrayList<Dimension>();
		Dimension dim = new Dimension();
		dim.setName("InstanceId");
		dim.setValue(instanceID);
		dimensions.add(dim);
		req.setDimensions(dimensions);
		
		Date now = new Date();
		Calendar cal = Calendar.getInstance();
		cal.setTime(now);
		cal.add(Calendar.HOUR, -1);
		Date before = cal.getTime();
		req.setStartTime(before);
		req.setEndTime(now);
		
		ArrayList<String> statistics = new ArrayList<String>();
		statistics.add("Maximum"); statistics.add("Average");
		req.setNamespace("AWS/EC2");
		req.setMetricName("NetworkIn");
		req.setStatistics(statistics);
		req.setPeriod(60);
		GetMetricStatisticsResult netInRes = cli.getMetricStatistics(req);
		
		req.setMetricName("NetworkOut");
		GetMetricStatisticsResult netOutRes = cli.getMetricStatistics(req);
		
		req.setMetricName("CPUUtilization");
		req.setUnit("Percent");
		GetMetricStatisticsResult cpuRes = cli.getMetricStatistics(req);
		
		List<Datapoint> dataRes = netInRes.getDatapoints();
		List<Datapoint> dataRes2 = netOutRes.getDatapoints();
		List<Datapoint> dataRes3 = cpuRes.getDatapoints();

		resultList.add(dataRes);
		resultList.add(dataRes2);
		resultList.add(dataRes3);

		return resultList;
	}
}
