package edu.columbia.cs.irt.sipcloud;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.regex.PatternSyntaxException;

/**
 * 
 */

/**
 * @author jyk
 *
 */
public class ConfigFileHandler {
	
	static String SP = " ";
	
	public boolean createDispatchListFile(String region, List<VmElement> vmList) {
		boolean isValidFile = false;
		try {
			File dispFile = new File(Constants.dispatchListFileName);
			FileWriter fstream = new FileWriter(dispFile, false);
			BufferedWriter bw = new BufferedWriter(fstream);	
		
			for(VmElement e: vmList) {
				if (e.type.compareTo(Constants.TYPE_SER) == 0 && e.processRunning == true && e.region.compareTo(region) == 0) { 
					bw.write("1 sip:" + e.privAddr + ":" + Constants.SER_PORT + "\n");
					isValidFile = true;
				}
			}
			
			bw.close(); 
		} catch (IOException e1) {
			e1.printStackTrace();
			return false;
		}
		return isValidFile;
	}
	
	public void buildCassConfigFile(String fileName, String seedAddress, boolean isFirstInstance) {
		try {
			// open template config file (read-only)
			File configTemplateFile = new File(Constants.localConfDir+Constants.cassConfigFileName+Constants.templateExtension);
			FileReader readStream = new FileReader(configTemplateFile);
			BufferedReader br = new BufferedReader(readStream);
			
			// open new config file (write-only)
			File configFile = new File(fileName);	
			FileWriter writeStream = new FileWriter(configFile, false);
			BufferedWriter bw = new BufferedWriter(writeStream);	
			
			// read each line, replace some strings
			String oneLine = null;
			String replacedLine = null;
			String seedPattern = "127.0.0.1";
			String bootstrapPattern = "auto_bootstrap: false";
			String bootstrapChanged = "auto_bootstrap: true";
			while ((oneLine = br.readLine()) != null) 
			{
				if (oneLine.indexOf(seedPattern) != -1) 
				{	// sed -i "s/127.0.0.1/$1/g"
					replacedLine = oneLine.replace(seedPattern, seedAddress);
				} 
				else if (oneLine.indexOf(bootstrapPattern) != -1 && !isFirstInstance) 
				{	// sed -i "s/auto_bootstrap: false/auto_bootstrap: true/g"
					replacedLine = oneLine.replace(bootstrapPattern, bootstrapChanged);
				} 
				else 
				{
					replacedLine = oneLine;	
				}
				bw.write(replacedLine + "\n");		// re-insert newline trimmed by br.readLine()
			}
			
			bw.close(); 
			br.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		} catch (PatternSyntaxException e2) {
			e2.printStackTrace();
		}
		
	}
	
	public void buildCass108ConfigFile(String fileName, String seedAddress, String privAddress, String pubAddress) {
		try {
			// open template config file (read-only)
			File configTemplateFile = new File(Constants.localConfDir+Constants.cass108ConfigFileName+Constants.templateExtension);
			FileReader readStream = new FileReader(configTemplateFile);
			BufferedReader br = new BufferedReader(readStream);
			
			// open new config file (write-only)
			File configFile = new File(fileName);	
			FileWriter writeStream = new FileWriter(configFile, false);
			BufferedWriter bw = new BufferedWriter(writeStream);	
			
			// read each line, replace some strings
			String oneLine = null;
			String replacedLine = null;
			String seedPattern = "SEED_ADDR";
			String privAddrPattern = "LISTEN_ADDR";
			String pubAddrPattern = "PUBLIC_ADDR";
			while ((oneLine = br.readLine()) != null) 
			{
				if (oneLine.indexOf(seedPattern) != -1) 
				{	// sed -i "s/127.0.0.1/$1/g"
					replacedLine = oneLine.replace(seedPattern, seedAddress);
				} 
				else if (oneLine.indexOf(privAddrPattern) != -1) 
				{	// sed -i "s/127.0.0.1/$1/g"
					replacedLine = oneLine.replace(privAddrPattern, privAddress);
				} 
				else if (oneLine.indexOf(pubAddrPattern) != -1) 
				{	// sed -i "s/127.0.0.1/$1/g"
					replacedLine = oneLine.replace(pubAddrPattern, pubAddress);
				} 
				else 
				{
					replacedLine = oneLine;	
				}
				bw.write(replacedLine + "\n");		// re-insert newline trimmed by br.readLine()
			}
			
			bw.close(); 
			br.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		} catch (PatternSyntaxException e2) {
			e2.printStackTrace();
		}
		
	}
	
	public void buildSerConfigFile(String fileName, String region, String listenAddress, String pubAddress, String cassAddress) {
		
		// preparation phase
		/// 1. get relay private addresses
		ArrayList<String> relayAddrList = LoadScalingManager.getPrivAddresses(region, Constants.TYPE_RELAY);
		String relayAddresses = buildRelayString(relayAddrList);
		if (relayAddresses == null) {
			System.err.println("No relay in the system! Abort buildSerConfigFile().");
			return;
		}
		
		/// 2. get cass private addresses
		ArrayList<String> cassAddrList = LoadScalingManager.getPrivAddresses(region, Constants.TYPE_CASS);
		String cassAddresses = buildCassString(cassAddrList);
		if (cassAddresses == null) {
			System.err.println("No cassandra in the system! Abort buildSerConfigFile().");
			return;
		}
		
		/// 3. choose a load balancer for out-going requests
		ArrayList<String> lbAddrList = LoadScalingManager.getPrivAddresses(region, Constants.TYPE_LB);
		String lbAddress = null;
		if (!lbAddrList.isEmpty()) {		// let's just do a random assignment
			Date d = new Date();
			int luckyWinner = (int) (d.getTime() % lbAddrList.size());
			lbAddress = lbAddrList.get(luckyWinner);
		} else {
			System.err.println("No load balancer in the system! Abort buildSerConfigFile().");
			return;
		}
		
		/// 4. get conference private addresses
		ArrayList<String> confAddrList = LoadScalingManager.getPrivAddresses(region, Constants.TYPE_CONF);
		int conf1 = 0;
		int conf2 = 1;
		if (confAddrList.isEmpty()) {
			System.err.println("No conference in the system! Abort buildSerConfigFile().");
			return;
		} 
		if (confAddrList.size() < 2) {
			System.err.println("Less than 2 conference servers in the system! No backup.");
			conf2 = 0;
		}
		
		
		// configuration phase
		try {
			// open template config file (read-only)
			File configTemplateFile = new File(Constants.localConfDir+Constants.serConfigFileName+Constants.templateExtension);
			FileReader readStream = new FileReader(configTemplateFile);
			BufferedReader br = new BufferedReader(readStream);
			
			// open new config file (write-only)
			File configFile = new File(fileName);	
			FileWriter writeStream = new FileWriter(configFile, false);
			BufferedWriter bw = new BufferedWriter(writeStream);	
			
			// read each line, replace some strings
			String oneLine = null;
			String replacedLine = null;
			
			String listenPattern = "LISTEN_ADDR";
			String pubPattern = "PUBLIC_ADDR";
			String cassPattern = "CASS_ADDR_LIST";
			String relayPattern = "RELAY_ADDR_LIST";
			String conference1Pattern = "CONFERENCE_ADDR1";
			String conference2Pattern = "CONFERENCE_ADDR2";
			String lbPattern = "LB_ADDR";

			while ((oneLine = br.readLine()) != null) 
			{
				if (oneLine.indexOf(listenPattern) != -1) 
				{	// sed -i "s/LISTEN_ADDR/$1/g"
					replacedLine = oneLine.replace(listenPattern, listenAddress + ":" + Constants.SER_PORT);
				} 
				else if (oneLine.indexOf(pubPattern) != -1) 
				{	// sed -i "s/PUBLIC_ADDR/$2/g"
					replacedLine = oneLine.replace(pubPattern, pubAddress);
				} 
				else if (oneLine.indexOf(cassPattern) != -1) 
				{	// sed -i "s/CASS_ADDR/$3/g"
					replacedLine = oneLine.replace(cassPattern, cassAddresses);
				} 
				else if (oneLine.indexOf(relayPattern) != -1) 
				{	// sed -i "s/RELAY_ADDR/$3/g"
					replacedLine = oneLine.replace(relayPattern, relayAddresses);
				} 
				else if (oneLine.indexOf(conference1Pattern) != -1) 
				{	// sed -i "s/CONFERENCE_ADDR1/$3/g"
					replacedLine = oneLine.replace(conference1Pattern, confAddrList.get(conf1) + ":" + Constants.conferencePort);
				} 
				else if (oneLine.indexOf(conference2Pattern) != -1) 
				{	// sed -i "s/CONFERENCE_ADDR2/$3/g"
					replacedLine = oneLine.replace(conference2Pattern, confAddrList.get(conf2) + ":" + Constants.conferencePort);
				} 
				else if (oneLine.indexOf(lbPattern) != -1) 
				{	// sed -i "s/LB_ADDR/$3/g"
					replacedLine = oneLine.replace(lbPattern, lbAddress);
				}  
				else 
				{
					replacedLine = oneLine;	
				}
				bw.write(replacedLine + "\n");		// re-insert newline trimmed by br.readLine()
			}
			
			bw.close(); 
			br.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		} catch (PatternSyntaxException e2) {
			e2.printStackTrace();
		}
	}
	
	public void buildLbConfigFile(String fileName, String listenAddress, String pubAddress) {
		try {
			// open template config file (read-only)
			File configTemplateFile = new File(Constants.localConfDir+Constants.lbConfigFileName+Constants.templateExtension);
			FileReader readStream = new FileReader(configTemplateFile);
			BufferedReader br = new BufferedReader(readStream);
			
			// open new config file (write-only)
			File configFile = new File(fileName);	
			FileWriter writeStream = new FileWriter(configFile, false);
			BufferedWriter bw = new BufferedWriter(writeStream);	
			
			// read each line, replace some strings
			String oneLine = null;
			String replacedLine = null;
			
			String listenPattern = "LISTEN_ADDR";
			String pubPattern = "PUBLIC_ADDR";

			while ((oneLine = br.readLine()) != null) 
			{
				if (oneLine.indexOf(listenPattern) != -1) 
				{	// sed -i "s/LISTEN_ADDR/$1/g"
					replacedLine = oneLine.replace(listenPattern, listenAddress);
				} 
				else if (oneLine.indexOf(pubPattern) != -1) 
				{	// sed -i "s/PUBLIC_ADDR/$2/g"
					replacedLine = oneLine.replace(pubPattern, pubAddress);
				} 
				else 
				{
					replacedLine = oneLine;	
				}
				bw.write(replacedLine + "\n");		// re-insert newline trimmed by br.readLine()
			}
			
			bw.close(); 
			br.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		} catch (PatternSyntaxException e2) {
			e2.printStackTrace();
		}
	}
	
	private String buildRelayString(ArrayList<String> relayAddrList) {
		String relayAddresses = null;
		for (String privAddr: relayAddrList) {
			if (relayAddresses == null)
				relayAddresses = "udp:" + privAddr + ":" + Constants.relayPort;
			else
				relayAddresses = relayAddresses + " udp:" + privAddr + ":" + Constants.relayPort;
		}
		return relayAddresses;
	}
	
	private String buildCassString(ArrayList<String> cassAddrList) {
		String cassAddresses = null;
		for (String privAddr: cassAddrList) {
			if (cassAddresses == null)
				cassAddresses = privAddr + ":" + Constants.cassPort;
			else
				cassAddresses = cassAddresses + "," + privAddr + ":" + Constants.cassPort;
		}
		return cassAddresses;
	}
	
	public void buildAsteriskSipConfigFile(String fileName, String pubAddress) {
		try {
			// open template config file (read-only)
			File configTemplateFile = new File(Constants.localConfDir+Constants.astSipConfigFileName+Constants.templateExtension);
			FileReader readStream = new FileReader(configTemplateFile);
			BufferedReader br = new BufferedReader(readStream);
			
			// open new config file (write-only)
			File configFile = new File(fileName);	
			FileWriter writeStream = new FileWriter(configFile, false);
			BufferedWriter bw = new BufferedWriter(writeStream);	
			
			// read each line, replace some strings
			String oneLine = null;
			String replacedLine = null;
			
			String pubPattern = "ASTERISK_PUB_ADDR";

			while ((oneLine = br.readLine()) != null) 
			{
				if (oneLine.indexOf(pubPattern) != -1) 
				{	// sed -i "s/ASTERISK_PUB_ADDR/$1/g"
					replacedLine = oneLine.replace(pubPattern, pubAddress);
				} 
				else 
				{
					replacedLine = oneLine;	
				}
				bw.write(replacedLine + "\n");		// re-insert newline trimmed by br.readLine()
			}
			
			bw.close(); 
			br.close();
		} catch (IOException e1) {
			e1.printStackTrace();
		} catch (PatternSyntaxException e2) {
			e2.printStackTrace();
		}
	}
	
	
}
