package edu.columbia.cs.irt.sipcloud;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;

import redstone.xmlrpc.XmlRpcException;
import redstone.xmlrpc.XmlRpcFault;
import redstone.xmlrpc.XmlRpcProxy;


/**
 * 
 */



/**
 * @author jyk
 *
 */
public class LoadMonitor extends Thread {
	
	boolean keepRunning = true;
	
	static interface overload
	{
	    public Integer get( String param ) throws XmlRpcFault;
	}
	
	ArrayList<overload> serRpcClients;
	ArrayList<overload> lbRpcClients;
	int nser;	// number of SER instances
	int nlb;	// number of LB instances
	

	public LoadMonitor(ArrayList<VmElement> vmList) {	
		serRpcClients = new ArrayList<overload>();
		lbRpcClients = new ArrayList<overload>();
		nser = 0;
		nlb = 0;
				
		for(VmElement e: vmList) {
			if (e.type.compareTo(Constants.TYPE_SER) !=0 
					&& e.type.compareTo(Constants.TYPE_LB) !=0) {
				continue;
			}
			
			URL url;
			String urlString;
			try {
				urlString = new String("http://" + e.pubAddr + ":" + Constants.SER_PORT);
				url = new URL(urlString);
			} catch (MalformedURLException e1) {
				System.err.println(e1);
				continue;
			}
			
			overload handler = (overload) XmlRpcProxy.createProxy(url, new Class[] { overload.class }, false);
			
			if (e.type.compareTo(Constants.TYPE_SER) == 0) {
				serRpcClients.add(handler);
				nser++;
			} else {
				lbRpcClients.add(handler);
				nlb++;
			}
		}
	}
	
	
	public void run() {
		ScaleCommand cmd = null;
		
		int serUpCounterMem = 0, serUpCounter = 0; 
		int serDnCounterMem = 0, serDnCounter = 0;
		int lbUpCounterMem = 0, lbUpCounter = 0;
		int lbDnCounterMem = 0, lbDnCounter = 0;
		
		while (keepRunning) {
			//System.out.println("Hello from Monitor thread!");
			
			int totalSerLoad = 0;
			for (overload handler: serRpcClients) {
				try {
					int load = (Integer) handler.get("request_rate");
					totalSerLoad += load;
					System.out.print("proxy_load = " + load + "\t");
				} catch (XmlRpcFault e) {
					System.err.println(e);
				} catch (XmlRpcException e) {
					System.err.println(e);
				}
			}
			//System.out.println();
			//System.out.println("nser = " + nser + ", totalSerLoad = " + totalSerLoad);
			
			
			if (nser == Constants.MAX_NUM_SER) {
			//	System.err.println("MAX_NUM_SER instances reached!");
			}
			if (nser < Constants.MAX_NUM_SER && totalSerLoad > Constants.MAX_LOAD_SER * (Constants.SCALE_UP_THRESHOLD_SER + nser-1 ) ) {
				serUpCounterMem = serUpCounter;
				serUpCounter++;
			//	System.out.print("Need to add a new SER instance! (" + serUpCounter + ") Target=");
			//	System.out.println(nser+1);
				if ( serUpCounter >= 5 ) {
					cmd = new ScaleCommand(Constants.CMD_ADD_PROXY, nser+1);
					//LoadScalingManager.putCommand(cmd);
					serUpCounter = serUpCounterMem = 0;
				}
				
			}
			if (nser > 1 && totalSerLoad < Constants.MAX_LOAD_SER * (Constants.SCALE_UP_THRESHOLD_SER + nser-2 ) ) {
				serDnCounterMem = serDnCounter;
				serDnCounter++;
			//	System.out.print("Need to remove a SER instance! (" + serDnCounter + ") Target=");
			//	System.out.println(nser-1);
				if ( serDnCounter >= 5 ) {
					cmd = new ScaleCommand(Constants.CMD_RMV_PROXY, nser-1);
					//LoadScalingManager.putCommand(cmd);
					serDnCounter = serDnCounterMem = 0; 
				}
			}
			if( serUpCounter == serUpCounterMem ) {
				serUpCounter = serUpCounterMem = 0;
			} else {
				serUpCounterMem = serUpCounter; 
			}
			if( serDnCounter == serDnCounterMem ) {
				serDnCounter = serDnCounterMem = 0; 
			} else {
				serDnCounterMem = serDnCounter;
			}
			
			int totalLbLoad = 0;
			for (overload handler: lbRpcClients) {
				try {
					totalLbLoad += (Integer) handler.get("request_rate");
				} catch (XmlRpcFault e) {
					System.err.println(e);
				} catch (XmlRpcException e) {
					System.err.println(e);
				}
			} 

			//System.out.println("nlb = " + nlb + ", totalLbLoad = " + totalLbLoad);
			
			if (nlb == Constants.MAX_NUM_LB) {
			//	System.err.println("MAX_NUM_LB instances reached!");
			}	
			if (nlb < Constants.MAX_NUM_LB && totalLbLoad > Constants.MAX_LOAD_LB * nlb) {
				lbUpCounterMem = lbUpCounter;
				lbUpCounter++;
			//	System.out.print("Need to add an LB instance! (" + lbUpCounter + ") Target=");
			//	System.out.println(nlb+1);
				if ( lbUpCounter >= 3 ) {
					cmd = new ScaleCommand(Constants.CMD_ADD_LB, nlb+1);
					//LoadScalingManager.putCommand(cmd);
					lbUpCounter = lbUpCounterMem = 0;
				}
			}
			if (nlb > 1 && totalLbLoad < Constants.MAX_LOAD_LB * (nlb - 1) ) {
				lbDnCounterMem = lbDnCounter;
				lbDnCounter++;
			//	System.out.print("Need to remove an LB instance! (" + lbDnCounter + ") Target=");
			//	System.out.println(nlb-1);
				if ( lbDnCounter >= 3 ) {
					cmd = new ScaleCommand(Constants.CMD_RMV_LB, nlb-1);
					//LoadScalingManager.putCommand(cmd);
					lbDnCounter = lbDnCounterMem = 0;
				}
			}
			if (lbUpCounter == lbUpCounterMem) {
				lbUpCounter = lbUpCounterMem = 0;
			}
			if (lbDnCounter == lbDnCounterMem) {
				lbDnCounter = lbDnCounterMem = 0;
			}

			
			try {
				Thread.sleep(Constants.POLL_INTERVAL);
			} catch (InterruptedException e) {
				System.err.println(e);
			}
		}
	}
}

