package edu.columbia.cs.irt.sipcloud;

import java.io.IOException;
import java.text.DateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;
import java.util.logging.FileHandler;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;
import java.util.concurrent.ConcurrentLinkedQueue;

import com.amazonaws.services.cloudwatch.AmazonCloudWatchClient;
import com.amazonaws.services.ec2.model.InstanceType;
import com.amazonaws.services.cloudwatch.model.Datapoint;


/**
 * @author jyk
 *
 */
public class LoadScalingManager extends Thread {
	
	boolean keepRunning = true;
	
	static AwsCenter aws;
	static ArrayList<VmElement> vmList; 
	static LoadMonitor monitor;
	static VmElement cassandraSeed;		
	static ConfigFileHandler fHandler;
	static Logger logger;
	static DnsHandler dnsHandler;
	static ConcurrentLinkedQueue<ScaleCommand> commands = new ConcurrentLinkedQueue<ScaleCommand>();
	
	

	public void init() {
		monitor = null;
		cassandraSeed = null;
		fHandler = new ConfigFileHandler();
		dnsHandler = new DnsHandler();
		
		// Setup logging
		logger = Logger.getLogger(Logger.GLOBAL_LOGGER_NAME);
		FileHandler fh = null;
		try {
			fh = new FileHandler(Constants.logFileName, true);
			SimpleFormatter sf = new SimpleFormatter();
			fh.setFormatter(sf);
		} catch (SecurityException e1) {
			e1.printStackTrace();
			return;
		} catch (IOException e1) {
			e1.printStackTrace();
			return;
		}
		logger.addHandler(fh);
		
		// Get existing instances 
		aws = new AwsCenter();
		vmList = new ArrayList<VmElement>();
		
		try {
			aws.init();
			aws.fillVmList(vmList);
		} catch (Exception e) {
			System.err.println("Error trying to get existing instances! Exiting program.");
			e.printStackTrace();
			System.exit(0);
		}

	}
	
	/**
	 * @param args
	 */
	public void run() {
		
		init();
		
		// manage cluster

		monitor = new LoadMonitor(vmList);
		//monitor.start();

		
		ScaleCommand cmd = null;
		while (keepRunning) {
			try {
				cmd = getCommand();
				if( cmd == null) { 
					Thread.sleep(1000);
					continue;
				}
				
				if (cmd.command.equals(Constants.CMD_ADD_PROXY) ) 
					;
					//addNewProxy();
				else if (cmd.command.equals(Constants.CMD_ADD_LB) )
					;
					//addNewLB();
				else if (cmd.command.equals(Constants.CMD_RMV_PROXY) )
					;
					//removeProxy();
				else if (cmd.command.equals(Constants.CMD_RMV_LB) )
					;
					//removeLB();
				
				
				//reset monitor
				monitor.keepRunning = false;
				monitor = new LoadMonitor(vmList);
				//monitor.start();

				
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		monitor.keepRunning = false;
	}
	
	public static void startProcess(VmElement e) {
		String type = e.type;
		String runCmd = null;
		String SP = " ";
		
		// tag it first
		aws.tagInstance(e);
		
		
		// upload config file and run whatever it is supposed to run
		String localFileName = null;
		String remoteFileName = null;
		
		if( type.compareTo(Constants.TYPE_CASS) == 0) {
			// same region - use seed's private address, diff region - use public address
			String seedAddr = (e.region.compareTo(cassandraSeed.region)==0)?cassandraSeed.privAddr:cassandraSeed.pubAddr;
			if (e.isSeed) seedAddr = cassandraSeed.pubAddr;
			
			localFileName = Constants.cassConfigFileName;
			remoteFileName = Constants.remoteCassConfigDirPath+Constants.cassConfigFileName;
			runCmd = "./apache-cassandra-1.0.8/bin/cassandra; sleep 100; " + 
					"apache-cassandra-1.0.8/bin/nodetool -h " + e.privAddr + " setcachecapacity SER credentials 200000 10000";
			fHandler.buildCass108ConfigFile(localFileName, seedAddr, e.privAddr, e.pubAddr);
		} 
		else if( type.compareTo(Constants.TYPE_SER) == 0) {
			localFileName = Constants.serConfigFileName;
			remoteFileName = Constants.remoteSerConfigDirPath+Constants.serConfigFileName;
			String memorySize = "";
			if (Constants.instanceType.equals(InstanceType.M1Large)) memorySize = "-m 4096";
			runCmd = ". .bash_profile; cd sip-router; ./ser " + memorySize + " -f ser.cfg; sleep 5; cd ..; ./use_dual_core.sh";
			fHandler.buildSerConfigFile(localFileName, e.region, e.privAddr, e.pubAddr, LoadScalingManager.cassandraSeed.privAddr);
		} 
		else if( type.compareTo(Constants.TYPE_LB) == 0) {
			localFileName = Constants.lbConfigFileName;
			remoteFileName = Constants.remoteLbConfigDirPath+Constants.lbConfigFileName;
			runCmd = ". .bash_profile; cd sip-router; ./ser -f ser-lb.cfg; sleep 5; cd ..; ./use_dual_core.sh";
			fHandler.buildLbConfigFile(localFileName, e.privAddr, e.pubAddr);
		}
		else if( type.compareTo(Constants.TYPE_RELAY) == 0) {
			runCmd = "sudo rtpproxy/bin/rtpproxy -u rtpproxy -A " + e.pubAddr + " -l " + e.privAddr + " -s udp:*:" + Constants.relayPort + " 2>&1; echo \"hello\"";
		}
		else if( type.compareTo(Constants.TYPE_CONF) == 0) {
			localFileName = Constants.localConfDir+Constants.astSipConfigFileName;
			remoteFileName = Constants.remoteAstConfigDirPath+Constants.astRemoteSipConfigFileName;
			runCmd = "cd /etc/asterisk/; " +
					"echo \"[conference]\" | cat >> extensions.conf; " +
					"echo \"exten => _XXX,1,Answer\" | cat >> extensions.conf; " +
					"echo \"exten => _XXX,n,MeetMe(,1cdM)\" | cat >> extensions.conf; " +
					"echo \"exten => _XXX,n,Hangup\" | cat >> extensions.conf; " +
					"/etc/init.d/asterisk restart";
			fHandler.buildAsteriskSipConfigFile(localFileName, e.pubAddr);
			//localFileName = Constants.localConfDir+Constants.astScriptFileName;
			//remoteFileName = Constants.remoteAstConfigDirPath+Constants.astScriptFileName;
			//runCmd = "chmod 755 "+remoteFileName+"; dos2unix "+remoteFileName+"; "+remoteFileName;   
		} 
		else {
			System.out.println("Unknown instance type! Do nothing.");
		}
		
		if (runCmd != null) {
			SshClient sshclient = new SshClient(e.pubAddr);
			if( type.compareTo(Constants.TYPE_CONF) == 0) {
				sshclient.setUser(Constants.confSshUser);
			}
			
			// upload config file and execute command
			if (sshclient.testConnection()) {
				sshclient.uploadFile(localFileName,remoteFileName);
				System.out.println("Remotely executing:\n"+runCmd);
				if (type.compareTo(Constants.TYPE_RELAY) == 0 || type.compareTo(Constants.TYPE_CONF) == 0) {
					sshclient.exec(runCmd,true);
				} else {
					sshclient.exec(runCmd);
				}
				e.processRunning = true;
			}
		}
		
	}
	
	private static void configureNewCassandraKeyspace(VmElement seed) {
		String runCmd = ". .bash_profile; " +
					 	"cd cass-client; ./create_ser_keyspace2 " + seed.privAddr + "; " + 
					 	"./adduser.sh zebra " + Constants.sipDomain + " " + seed.privAddr + "; " +
					 	"./adduser.sh aardvark " + Constants.sipDomain + " " + seed.privAddr;
		
		SshClient sshclient = new SshClient(seed.pubAddr);	
		if (sshclient.testConnection()) {
			System.out.println("Remotely executing:\n"+runCmd);
			sshclient.exec(runCmd);
		}
	}
	
	private static void updateCassReplicationFactor(VmElement seed, int replicationFactor) {
		String runCmd = ". .bash_profile; " + 
						"echo \"update keyspace SER with strategy_options={replication_factor:" + replicationFactor + "}\" > update_rep_factor.txt; " +
			 			"apache-cassandra-1.0.8/bin/cassandra-cli -h " + seed.privAddr + " -k SER -f ~/update_rep_factor.txt";
		SshClient sshclient = new SshClient(seed.pubAddr);	
		if (sshclient.testConnection()) {
			System.out.println("Remotely executing:\n"+runCmd);
			sshclient.exec(runCmd);
		}
	}
	
	private static void repairCassandraKeyspace() {
		String runCmd = null;
		for(VmElement e: vmList) {
			if (e.type.compareTo(Constants.TYPE_CASS) == 0) {
				runCmd = ". .bash_profile; " +
			 			"apache-cassandra-1.0.8/bin/nodetool -h " + e.privAddr + " repair SER";
				SshClient sshclient = new SshClient(e.pubAddr);	
				if (sshclient.testConnection()) {
					System.out.println("Remotely executing:\n"+runCmd);
					sshclient.exec(runCmd);
				}
			}
		}
	}
	
	private static int cassClusterSize() {
		int i = 0;
		for(VmElement e: vmList) {
			if (e.type.compareTo(Constants.TYPE_CASS) == 0) {
				i++;
			}
		}		
		return i;
	}
	
	
	public static void startNewCluster(String region) {
		// Reset DNS records
		clearDNS(region);
		

		// Add a cassandra instance
		addNewCassandra(region);
		addNewCassandra(region);
		updateCassReplicationFactor(cassandraSeed, cassClusterSize());
		repairCassandraKeyspace();
		

		

		/*
		// Add a relay instance
		addNewRelay(region);
		addNewRelay(region);
		
		// Add a conference server
		addNewConference(region);
		addNewConference(region);
		
		// Add an LB instance
		addNewLB(region);
		addNewLB(region);
				
		// Add a SER instance
		addNewProxy(region);
		addNewProxy(region);
		*/
	}


	public static boolean uploadDispatchListFile(String host) {
		boolean status = false;
		SshClient sshclient = new SshClient(host);
		if (sshclient.testConnection()) {
			status = sshclient.uploadFile(Constants.dispatchListFileName, Constants.remoteLbConfigDirPath+Constants.dispatchListFileName);
		} 
		return status;
	}

	/**
	 * Updates all LBs with new dispatch list file.
	 */
	public static void updateAllDispatchList(String region) {
		for(VmElement e: vmList) {
			if (e.type.compareTo(Constants.TYPE_LB) == 0 && e.region.compareTo(region) == 0) {
				if( !uploadDispatchListFile(e.pubAddr) ) {
					System.err.println("Upload dispatch file to " + e.pubAddr + " failed!");
					continue;
				}
				// TODO: need to update LB using (sercmd mi ds_reload)
				SshClient sshclient = new SshClient(e.pubAddr);
				String configCmd = ". .bash_profile; cd sip-router/utils/sercmd; ./sercmd -s tcp:" + e.privAddr + ":20460 mi ds_reload";
				if (sshclient.testConnection()) {
					sshclient.exec(configCmd);
				}
				System.out.println("---------------- Updated dispatcher list on " + e.pubAddr + " --------------");
				
				/*
				SshClient sshclient = new SshClient(e.pubAddr);
				String configCmd = ". .bash_profile; cd sip-router/utils/sercmd; ./sercmd -s tcp:" + e.privAddr + ":20460 mi ds_reload; ./sercmd -s tcp:" + e.privAddr + ":20460 mi ds_set_state i 1 sip:" + ser.privAddr + ":5062";
				sshclient.exec(configCmd);
				System.out.println("---------------- Updated dispatcher list on " + e.pubAddr + " --------------");
				sshclient.exec(". .bash_profile; cd sip-router/utils/sercmd; ./sercmd -s tcp:" + e.privAddr + ":20460 mi ds_set_state a 1 sip:" + ser.privAddr + ":5062");
				*/
				
			}
		}
	}
	
	/**
	 * Updates all LBs to inactivate proxy
	 */
	public static void inactivateProxy(VmElement ser) {
		for(VmElement e: vmList) {
			if (e.type.compareTo(Constants.TYPE_LB) == 0 && e.region.compareTo(ser.region) == 0) {
				// TODO: need to update LB using (sercmd mi ds_reload)
				SshClient sshclient = new SshClient(e.pubAddr);
				String configCmd = ". .bash_profile; cd sip-router/utils/sercmd; ./sercmd -s tcp:" + e.privAddr + ":20460 mi ds_set_state i 1 sip:" + ser.privAddr + ":5062";
				if (sshclient.testConnection()) {
					sshclient.exec(configCmd);
				}
			}
		}
	}
	
	public static void addNewProxy(String region) {
		System.out.println("MAIN: addProxy called!");

		Date startDate = new Date();
		DateFormat df = DateFormat.getDateTimeInstance();
		df.setTimeZone(TimeZone.getTimeZone("UTC"));
		LoadScalingManager.logger.info("Proxy start decision granted: " + df.format(startDate) + "  -------------  ");
		
		// Add a SER instance
		VmElement ser = aws.addInstance(region, Constants.TYPE_SER);
		vmList.add(ser);
		startProcess(ser);
		
		reconfigureLB(region);
		
		Date launchDate = new Date();
		LoadScalingManager.logger.info("Proxy launch time: " + df.format(launchDate) + ser.instanceID + " --------------- ");
	}
	
	public static void removeRandomProxy(String region) {
		System.out.println("MAIN: removeProxy called!");
		
		VmElement ser = null;
		
		// Pick a SER instance
		for(VmElement e: vmList) {
			if (e.type.compareTo(Constants.TYPE_SER) == 0 && e.region.compareTo(region) == 0) {
				vmList.remove(e);
				ser = e;
				break;
			}
		}
		
		System.err.println("Will remove SER instance ID: " + ser.instanceID + "...");
		
		Date startDate = new Date();
		DateFormat df = DateFormat.getDateTimeInstance();
		df.setTimeZone(TimeZone.getTimeZone("UTC"));
		LoadScalingManager.logger.info("Proxy remove time: " + df.format(startDate) + "  -------------  " + ser.instanceID);
		
		// Make SER instance inactive
		inactivateProxy(ser);
		
		// Give time for instance to finish calls.
		try {
			Thread.sleep(100000);		// 10 seconds
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}
		
		// Remove the SER instance 
		aws.removeInstance(ser);
		
		reconfigureLB(ser.region);
	}
	
	private static void reconfigureLB(String region) {
		// Remove SER from dispatcher list and upload new dispatcher list 
		if (!fHandler.createDispatchListFile(region, vmList)) {
			System.out.println("Something wrong with dispatcher list file. Run LB instance manually!");
			return;
		}
		
		updateAllDispatchList(region);
	}
	
	public static void addNewLB(String region) {
		System.out.println("MAIN: addNewLB called!");
		
		VmElement lb = aws.addInstance(region, Constants.TYPE_LB);
		vmList.add(lb);
		
		// Upload dispatcher list if it exists
		if (!uploadDispatchListFile(lb.pubAddr)) {
			System.out.println("Cannot upload dispatch file. Maybe you're starting a cluster? If not, upload it manually!");
			aws.tagInstance(lb);
		}
		startProcess(lb);

		
		// Update DNS server
		// 1. name the new load balancer: ex) 128-59-18-73.sip.epic911.net
		String name = dnsHandler.nameOfLB(lb.pubAddr);
		// 2. add it to the DNS server
		dnsHandler.addLb(name, lb.pubAddr, Constants.SER_PORT);
	}
	
	public static void removeLB() {
		System.out.println("MAIN: removeLB called!");
		
		// TODO: don't forget to remove DNS record
	}
	
	public static void removeCassandra() {
		System.out.println("MAIN: removeLB called!");
		
		// TODO: don't forget to remove DNS record
	}
	
	
	public synchronized static void putCommand(ScaleCommand command) {
		commands.add(command);
	}
	
	public synchronized static ScaleCommand getCommand() {
		ScaleCommand newCommand = commands.poll();
		
		// commands is empty
		if (newCommand == null)		
			return null;
		
		// remove all identical commands in queue
		while (true) {
			if (commands.peek() == null)
				break;
			if (commands.peek().command.equals(newCommand.command)) {
				commands.poll();
			} else {
				break;
			}
		}
		
		// if command has been fulfilled, return null
		if( newCommand.command.compareTo(Constants.CMD_ADD_PROXY) == 0 || 
				newCommand.command.compareTo(Constants.CMD_RMV_PROXY) ==0 ) {
			int i = 0;
			for(VmElement e: vmList) {
				if (e.type.compareTo(Constants.TYPE_SER) == 0) {
					i++;
				}
			}

			if ( newCommand.command.equals(Constants.CMD_ADD_PROXY)  && i >= newCommand.nthInstance ) {
				return null;
			} else if ( newCommand.command.equals(Constants.CMD_RMV_PROXY)  && i <= newCommand.nthInstance ) {
				return null;
			}
		}
		
		if( newCommand.command.equals(Constants.CMD_ADD_LB)  || 
				newCommand.command.equals(Constants.CMD_RMV_LB)  ) {
			int i = 0;
			for(VmElement e: vmList) {
				if (e.type.compareTo(Constants.TYPE_LB) == 0) {
					i++;
				}
			}
			
			if( newCommand.command.equals(Constants.CMD_ADD_LB)  && i >= newCommand.nthInstance) {
				return null;
			} else if ( newCommand.command.equals(Constants.CMD_RMV_LB)  && i <= newCommand.nthInstance) {
				return null;
			}
		}
		
		return newCommand;
	}

	public static void addNewVms(String region, boolean[] selectedVms) {
		for (int i = selectedVms.length-1; i >= 0; i-- ) {		// start VMs in proper order
			if ( selectedVms[i] ) {
				try {
					switch (Constants.VmTypes.getVmType(i)) {
					case LB:
						addNewLB(region);
						System.out.println("Start LB");
						break;
					case PROXY:
						addNewProxy(region);
						System.out.println("Start PROXY");
						break;
					case DB:
						addNewCassandra(region);
						System.out.println("Start DB");
						break;
					case RELAY:
						addNewRelay(region);
						System.out.println("Start RELAY");
						break;
					case CONFERENCE:
						addNewConference(region);
						System.out.println("Start CONFERENCE");
						break;
					}
				} catch (Exception e) {
					System.err.println(e);
				}
			}
		}
		
	}

	private static void addNewConference(String region) {
		VmElement conf = aws.addInstance(region, Constants.TYPE_CONF);
		vmList.add(conf);
		startProcess(conf);
	}

	private static void addNewRelay(String region) {
		VmElement relay = aws.addInstance(region, Constants.TYPE_RELAY);
		vmList.add(relay);
		startProcess(relay);
	}

	private static void addNewCassandra(String region) {
		VmElement cass = aws.addInstance(region, Constants.TYPE_CASS);
		if (cassandraSeed == null) {
			cass.isSeed = true;
			cassandraSeed = cass;
		}
		vmList.add(cass);
		aws.updateSecurityGroup(cass);	// update security group for cross-region clustering
		startProcess(cass);
		
		if (cass.isSeed) {
			configureNewCassandraKeyspace(cass);
		}
	}
	
	public static void stopCluster(String region) {
		clearDNS(region);
		for(VmElement e: vmList) {
			if (e.region.compareTo(region) == 0)
				aws.removeInstance(e);
		}
		if (vmList.isEmpty()) {		// everything is gone!
			cassandraSeed = null;
		}
	}

	private static void clearDNS(String region) {
		// Erase all DNS records
		for(VmElement e: vmList) {
			if (e.type.compareTo(Constants.TYPE_LB) == 0 && e.region.compareTo(region)==0) {
				clearDNS(e);
			}
		}
	}
	
	private static void clearDNS(VmElement lb) {
		String name = dnsHandler.nameOfLB(lb.pubAddr);
		dnsHandler.removeLb(name);
	}
	
	public static ArrayList<String> getPrivAddresses(String region, String type) {
		ArrayList<String> privAddrList = new ArrayList<String>();	
		for(VmElement e: vmList) {
			if (e.type.compareTo(type) == 0 && e.region.compareTo(region) == 0) {
				privAddrList.add(e.privAddr);
			}
		}
		return privAddrList;
	}
	
	public static String getPubAddressFromPrivateAddress(String privAddress) {
		String pubAddr = null;	
		for(VmElement e: vmList) {
			if (e.privAddr.compareTo(privAddress) == 0) {
				pubAddr = e.pubAddr;
			}
		}
		return pubAddr;
	}
	
	
	public static void stopInstance(String comboSelection) {
		String[] fields = comboSelection.split(" ");
		String instanceID = fields[0];
		stopInstanceUsingID(instanceID);
	}
	
	public static void stopInstanceUsingID(String instanceID) {
		String stopCmd = null;
		VmElement selected = null;
		
		for(VmElement e: vmList) {
			if (e.instanceID.compareTo(instanceID) == 0) {
				selected = e;
				break;
			}
		}
		
		if (selected == null) {
			System.err.println("InstanceID " + instanceID + " is not found in VmList: cannot stop instance.");
			return;
		}

		if (selected.type.compareTo(Constants.TYPE_CASS) == 0) {
			stopCmd = "killall java";
		} else if (selected.type.compareTo(Constants.TYPE_RELAY) == 0) {
			stopCmd = "sudo killall rtpproxy";
		} else if (selected.type.compareTo(Constants.TYPE_CONF) == 0) {
			stopCmd = "killall asterisk";
		} else if (selected.type.compareTo(Constants.TYPE_LB) == 0) {
			stopCmd = "killall ser";
		} else if (selected.type.compareTo(Constants.TYPE_SER) == 0) {
			stopCmd = "killall ser";
		} 
		
		if (stopCmd != null) {
			SshClient sshclient = new SshClient(selected.pubAddr);
			if( selected.type.compareTo(Constants.TYPE_CONF) == 0) {
				sshclient.setUser(Constants.confSshUser);
			}
			
			// execute command
			if (sshclient.testConnection()) {
				System.out.println("Remotely executing:\n"+stopCmd);
				if (selected.type.compareTo(Constants.TYPE_RELAY) == 0) {
					sshclient.exec(stopCmd,true);
				} else {
					sshclient.exec(stopCmd);
				}
				selected.processRunning = false;
				reconfigureAfterInstanceFailure(selected);
			}
		}
	}
	
	/**
	 * Terminate EC2 Instance
	 * 
	 * @param instanceID The VM's instance ID to be terminated
	 */
	public static void terminateInstanceUsingID(String instanceID) {
		System.err.println("Now VM "+ instanceID + " will terminated.");
		VmElement selected = null;
		
		for(VmElement e: vmList) {
			if (e.instanceID.compareTo(instanceID) == 0) {
				selected = e;
				break;
			}
		}
		
		if (selected == null) {
			System.err.println("InstanceID " + instanceID + " is not found in VmList: cannot terminate instance.");
			return;
		}
		
		aws.removeInstance(selected);
		vmList.remove(selected);
	}
	
	/**
	 * Re-config EC2 Instance
	 * 
	 * @param instnaceID the VM's instance ID to be re-configured
	 */
	public static void reconfigInstanceUsingID(String instanceID) {
		System.out.println("Now re-configure the VM " + instanceID);
	}
	
	private static void reconfigureAfterInstanceFailure(VmElement failedVm) {
		
		if (failedVm.type.compareTo(Constants.TYPE_LB) == 0) {
			clearDNS(failedVm);
		} else if (failedVm.type.compareTo(Constants.TYPE_SER) == 0) {
			reconfigureLB(failedVm.region);
		} 
	}
	
	public static String[] makeListOfInstances() {
		String[] list;
		if (vmList == null) {
			list = new String[0];
			return list;
		}
		else
			list = new String[vmList.size()];

		int i = 0;
		for(VmElement e: vmList) {
			list[i] = e.instanceID + " " + e.type + " " + e.pubAddr + " " + e.privAddr + " " + e.region + " " + e.isSeed + " " + e.processRunning;
			i++;
		}
		return list;
	}
	
	public static ArrayList<List<Datapoint>> getVMStatistic(String instanceID) {
		return aws.getVMStatistic(instanceID);
	}
	
}