/**
 * 
 */
package edu.columbia.cs.irt.sipcloud;

import com.jcraft.jsch.*;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.ConnectException;
import java.util.Properties;
import java.util.Vector;

/**
 * @author jyk
 *
 */
public class SshClient {
	
	String privKeyFilePath = Constants.privKeyFile;
	String user = Constants.sshUser;
	//String privKeyFilePath = "id_rsa";
	//String user = "jyk";
	String host;
	int port = 22;
	Properties config;
	
	public SshClient(String host) {
		this.host = host;
		config = new Properties();
		config.put("StrictHostKeyChecking", "no");
		System.out.println("Key file path="+privKeyFilePath);
	}
	
	public void setUser(String user) {
		this.user = user;
	}
	
	// test connection before doing real work. Test connection every 10 seconds for a minute.
	public boolean testConnection() {
		boolean isConnected = false;
		int retryCount = 0;
		
		while (retryCount < 12) {
			int status = exec("echo \"testConnection() success!\"");
			if (  status > 0 ) {
				isConnected = true;
				break;
			} else if ( status == -500 ){		// connection refused. retry.
				retryCount++;
				try {
					Thread.sleep(10000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			} else {							// some other error. stop trying. 
				break;
			}
		}
		
		return isConnected;
	}
	
	public int exec(String command) {
		return exec(command,false);
	}
	
	public int exec(String command, boolean enablePty) {
		int exitStatus = -100; 
		
		try {
			//JSch.setLogger(new MyLogger());
			JSch jsch = new JSch();
			//jsch.removeAllIdentity();
			jsch.addIdentity(privKeyFilePath);
			
			/*
			Vector v = jsch.getIdentityNames();
			for (int i = 0; i < v.size(); i++) {
				System.out.println(v.get(i));
			}
			*/
			
			
			Session session = jsch.getSession(user, host, port);
			session.setConfig(config);
			session.connect();
			
			Channel channel = session.openChannel("exec");
			((ChannelExec)channel).setCommand(command);
			((ChannelExec)channel).setPty(enablePty);
	
			channel.setInputStream(null);
			
			/*
			channel.setOutputStream(System.out);
			//((ChannelExec)channel).setErrStream(System.err);
			StreamGobbler errorGobbler = new StreamGobbler(((ChannelExec)channel).getErrStream(), "ERROR");
			errorGobbler.start();
			*/
			
			channel.connect();
			InputStream in=channel.getInputStream();
	
			byte[] tmp=new byte[1024];
			while(true){
				while(in.available()>0) {
					int i=in.read(tmp, 0, 1024);
					if(i<0)break;
					System.out.print(new String(tmp, 0, i));
				}
				if(channel.isClosed()) {
					exitStatus = channel.getExitStatus();
					System.out.println("exit-status: "+ exitStatus);
					break;
				}
				try{Thread.sleep(1000);}catch(Exception ee){}
			}

			channel.disconnect();
			session.disconnect();
			
			exitStatus = 100;
			
		} catch (JSchException e1) {
			System.err.println(e1);
			if ( e1.getCause() instanceof java.net.ConnectException) {
				exitStatus = -500;
			} else {
				e1.printStackTrace();
			}
		} catch (Exception e2) {
			System.err.println(e2);
		}
		
		return exitStatus;
	}
	
	public boolean uploadFile(String localFilePath, String remoteFilePath) {	
		if ( localFilePath == null || remoteFilePath == null) 	return false;
		
		try {
			JSch jsch = new JSch();
			jsch.addIdentity(privKeyFilePath);
			Session session = jsch.getSession(user, host, port);
			session.setConfig(config);
			session.connect();
			
			String command = "scp -p -t " + remoteFilePath;
			Channel channel=session.openChannel("exec");
		    ((ChannelExec)channel).setCommand(command);

		    // get I/O streams for remote scp
			OutputStream out=channel.getOutputStream();
			InputStream in=channel.getInputStream();
			channel.connect();
			
			if(checkAck(in)!=0){
				return false;
			}
			
			// send "C0644 filesize filename", where filename should not include '/'
			long filesize=(new File(localFilePath)).length();
			command="C0644 "+filesize+" ";
			if(localFilePath.lastIndexOf('/')>0){
				command+=localFilePath.substring(localFilePath.lastIndexOf('/')+1);
			}
			else{
				command+=localFilePath;
			}
			command+="\n";
			out.write(command.getBytes()); out.flush();
			if(checkAck(in)!=0){
				return false;
			}
			
			// send a content of lfile
			FileInputStream fis=new FileInputStream(localFilePath);
			byte[] buf=new byte[1024];
			while(true) {
			  int len=fis.read(buf, 0, buf.length);
			if(len<=0) break;
			  out.write(buf, 0, len); //out.flush();
			}
			fis.close();
			fis=null;
			
			// send '\0'
			buf[0]=0; out.write(buf, 0, 1); out.flush();
			if(checkAck(in)!=0){
				return false;
			}
			out.close();
			
			channel.disconnect();
			session.disconnect();

		} catch (JSchException e1) {
			System.err.println(e1);
			return false;
		} catch (Exception e2) {
			System.err.println(e2);
			return false;
		}
		
		return true;
	}
	
	private int checkAck(InputStream in) throws IOException{
	    int b=in.read();
	    // b may be 0 for success,
	    //          1 for error,
	    //          2 for fatal error,
	    //          -1
	    if(b==0) return b;
	    if(b==-1) return b;

	    if(b==1 || b==2){
	      StringBuffer sb=new StringBuffer();
	      int c;
	      do {
		c=in.read();
		sb.append((char)c);
	      }
	      while(c!='\n');
	      if(b==1){ // error
		System.out.print(sb.toString());
	      }
	      if(b==2){ // fatal error
		System.out.print(sb.toString());
	      }
	    }
	    return b;
	  }
	
	
	
	public static class MyLogger implements com.jcraft.jsch.Logger {
	    static java.util.Hashtable name=new java.util.Hashtable();
	    static{
	      name.put(new Integer(DEBUG), "DEBUG: ");
	      name.put(new Integer(INFO), "INFO: ");
	      name.put(new Integer(WARN), "WARN: ");
	      name.put(new Integer(ERROR), "ERROR: ");
	      name.put(new Integer(FATAL), "FATAL: ");
	    }
	    public boolean isEnabled(int level){
	      return true;
	    }
	    public void log(int level, String message){
	      System.err.print(name.get(new Integer(level)));
	      System.err.println(message);
	    }
	  }

}
