package edu.columbia.cs.irt.sipcloud.gui;

import java.util.ArrayList;
import java.util.Hashtable;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;


import edu.columbia.cs.irt.sipcloud.Constants;
import edu.columbia.cs.irt.sipcloud.LoadScalingManager;

/**
 * MainWindow Canvas Update Thread
 * 
 * @author Jin Hyung Park [jp2105]
 *
 */
class CanvasUpdater implements Runnable {
	private Shell shell;
	
	private boolean isFirstRun;
	private boolean isDisposed;
	
	/* 
	 * Dictionary to store VM icons with key-value pair
	 * we can retrieve the VM icon by VmElement's TYPE
	 */
	private Hashtable<String, Image> vmImages;
	
	private String[] vmListStrings;
	private ArrayList<Label> vmLabelLists;
	private ArrayList<Label> vmLabelInfoLists;
	private FontData[] fD;
	
	/*
	 * tabbarGroup is tabItem to show each AWS region.
	 */
	private ArrayList<TabbarDataCenter> tabbarGroup;
	
	private MenuController menuController;

	/**
	 * CanvasUpdater Constructor Method
	 * 
	 * @param shell MainWindow's shell
	 * @param tabFolder MainWindow's TabFolder object
	 */
	public CanvasUpdater(Shell shell, TabFolder tabFolder) {
		this.shell = shell;
		isDisposed = false;
		
		/*
		 * Initialize each data center's tab
		 */
		tabbarGroup = new ArrayList<TabbarDataCenter>();
		TabbarDataCenter tmpTabbarDataCenter = new TabbarDataCenter(tabFolder, Constants.ec2UsEast);
		tabbarGroup.add(tmpTabbarDataCenter);
		tmpTabbarDataCenter = new TabbarDataCenter(tabFolder, Constants.ec2UsWest);
		tabbarGroup.add(tmpTabbarDataCenter);
		
		vmLabelLists = new ArrayList<Label>();
		vmLabelInfoLists = new ArrayList<Label>();
		
		/*
		 * Initialize images for each canvas
		 */
		Image vmProxy = new Image(shell.getDisplay(), "res/ser.png");
		Image vmLb = new Image(shell.getDisplay(), "res/lb.png");
		Image vmDb = new Image(shell.getDisplay(), "res/cassandra.png");
		Image vmRelay = new Image(shell.getDisplay(), "res/relay.png");
		Image vmConf = new Image(shell.getDisplay(), "res/conf.png");
		Image vmDbSeed = new Image(shell.getDisplay(), "res/cassandra_seed.png");
		Image vmNA = new Image(shell.getDisplay(), "res/type_na.png");
		
		/*
		 * Initialize Hashtables for Icons
		 */
		vmImages = new Hashtable<String, Image>();
		vmImages.put(Constants.TYPE_SER, vmProxy);
		vmImages.put(Constants.TYPE_LB, vmLb);
		vmImages.put(Constants.TYPE_CASS, vmDb);
		vmImages.put(Constants.TYPE_RELAY, vmRelay);
		vmImages.put(Constants.TYPE_CONF, vmConf);
		vmImages.put(Constants.TAG_SEED, vmDbSeed);
		vmImages.put(Constants.TYPE_NA, vmNA);
		
		menuController = new MenuController();
		
		isFirstRun = false;
	}
	
	/**
	 * Canvas Update Main Thread
	 * Here, we draw each VM's status on MainWindow.
	 * 
	 */
	public void run() {
		while (!isDisposed) {
			/*
			 * The VM icon update thread polls VM's status every 1 seconds.
			 */
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			/*
			 * If there are no running VMs, just keep looping.
			 */
			String[] vmListStringsTemp = LoadScalingManager.makeListOfInstances();
			if (vmListStringsTemp == null)
				continue;
			
			/*
			 * Receiving VM's info is same as the previous info, just keep looping.
			 * Otherwise, we update each canvas.
			 */
			if (vmListStrings != null && vmListStrings.length == vmListStringsTemp.length) {
				boolean isAllSame = true;
				for (int i = 0; i < vmListStringsTemp.length; ++i) {
					if (!vmListStrings[i].equals(vmListStringsTemp[i])) {
						System.out.println("[" + vmListStrings[i] + "] vs [" + vmListStringsTemp[i] + "]");
						isAllSame = false;
					}
				}
				
				if (isAllSame)
					continue;
			}
			
			/*
			 * If there are no any VMs on received information, 
			 * just keep looping until we receive the notification we receive new one. 
			 */
			vmListStrings = vmListStringsTemp;
			if (vmListStrings.length == 0 && isFirstRun == false)
				continue;
			
			isFirstRun = true;
			/* 
			 * if the number of VM is 0, and isFirstRun flag is true,
			 * all existed VMs are deleted. So, now we are back to the initial state.
			 * 
			 */
			if (vmListStrings.length == 0 && isFirstRun == true)
				isFirstRun = false;
			
			/* TODO: remove later. this is for debugging */
			for (int i = 0; i < vmListStrings.length; ++i)
				System.out.println("[" + i + "]" + vmListStrings[i]);
			
			/*
			 * Clear the previous information to save the new information.
			 */
			for (final Label label: vmLabelLists) {
				Display display = Display.getDefault();
				display.asyncExec(new Runnable() {
					public void run() {
						label.dispose();
					}
				}
				);
			}
			for (final Label label: vmLabelInfoLists) {
				Display display = Display.getDefault();
				display.asyncExec(new Runnable() {
					public void run() {
						label.dispose();
					}
				}
				);
			}
			vmLabelLists.clear();
			vmLabelInfoLists.clear();
			
			/*
			 * Now, display each VM information on each tab and canvas.
			 */
			shell.getDisplay().asyncExec(new Runnable() {
				@Override
				public void run() {
					int lb_n[] = {0, 0}, proxy_n[] = {0, 0}, db_n[] = {0, 0};
					int media_n[] = {0, 0}, conf_n[] = {0, 0}, temp_n[] = {0, 0}; 
					int x = 0, y = 0, icon_height = 95, pix_bt_ic_lb = 9;
					String infoString = null;
					for (int i = 0; i < vmListStrings.length; ++i) {
						Label label = null;
						Label infoLabel = null;
						String []vmInfo = vmListStrings[i].split(" ");
						TabbarDataCenter tDC = null;
						int isEast = 0;
						
						/* Find appropriate Tabbar for current VM */
						for (int j = 0; j < tabbarGroup.size(); j++) {
							tDC = tabbarGroup.get(j);
							if (vmInfo[4].equalsIgnoreCase(tDC.getTabName()))
								break;
						}
						
						/*
						 * Set VM's icon
						 */
						y = 25;
						System.out.println(vmInfo[0] + "::" + vmInfo[1]);
						if (vmInfo[4].contains("east"))
							isEast = 1;
						else
							isEast = 0;
						
						if (vmInfo[1].equals(Constants.TYPE_LB)) {
							x = 50;
							y = y + icon_height * lb_n[isEast];
							lb_n[isEast] = lb_n[isEast] + 1;
							infoString = "Load Balancer\nIP:" + vmInfo[2] + "\nPrv:" + vmInfo[3];
						} else if (vmInfo[1].equals(Constants.TYPE_SER)) {
							x = 200;
							y = y + icon_height * proxy_n[isEast];
							proxy_n[isEast] = proxy_n[isEast] + 1;
							infoString = "Proxy\nIP:" + vmInfo[2] + "\nPrv:" + vmInfo[3];
						} else if (vmInfo[1].equals(Constants.TYPE_CASS)) {
							x = 350;
							y = y + icon_height * db_n[isEast];
							db_n[isEast] = db_n[isEast] + 1;
							infoString = "Database\nIP:" + vmInfo[2] + "\nPrv:" + vmInfo[3];
						} else if (vmInfo[1].equals(Constants.TYPE_RELAY)) {
							/* Media Server & Conference Server will be on 2nd Canvas(Group) */
							x = 50;
							y = y + icon_height * media_n[isEast];
							media_n[isEast] = media_n[isEast] + 1;
							infoString = "Media Relay\nIP:" + vmInfo[2] + "\nPrv:" + vmInfo[3];
						} else if (vmInfo[1].equals(Constants.TYPE_CONF)) {
							x = 200;
							y = y + icon_height * conf_n[isEast];
							conf_n[isEast] = conf_n[isEast] + 1;
							infoString = "Conference\nIP:" + vmInfo[2] + "\nPrv:" + vmInfo[3];
						} else if (vmInfo[1].equals(Constants.TYPE_NA)) {
							x = 50;
							y = y + icon_height * temp_n[isEast];
							temp_n[isEast] = temp_n[isEast] + 1;
							infoString = "Initilizing VM";
						}

						/*
						 * Set VM's basic information
						 * i.e. Public IP, Private IP
						 */
						if (vmInfo[1].equals(Constants.TYPE_LB) || vmInfo[1].equals(Constants.TYPE_SER) || 
								vmInfo[1].equals(Constants.TYPE_CASS)) {
							label = new Label(tDC.canvas_1, SWT.IMAGE_PNG);
							infoLabel = new Label(tDC.canvas_1, SWT.LEFT);
						} else if (vmInfo[1].equals(Constants.TYPE_RELAY) || vmInfo[1].equals(Constants.TYPE_CONF)) {
							label = new Label(tDC.canvas_2, SWT.IMAGE_PNG);
							infoLabel = new Label(tDC.canvas_2, SWT.LEFT);
						} else if (vmInfo[1].equals(Constants.TYPE_NA)) {
							label = new Label(tDC.canvas_3, SWT.IMAGE_PNG);
							infoLabel = new Label(tDC.canvas_3, SWT.LEFT);
						}
						
						// Set VM Icon and VM Label position
						if (vmInfo[5].equalsIgnoreCase("true"))
							label.setImage(vmImages.get(Constants.TAG_SEED));
						else
							label.setImage(vmImages.get(vmInfo[1]));
						label.pack();
						label.setLocation(x, y + pix_bt_ic_lb);

						/* 
						 * Set Right-click popup menu
						 */
						menuController.makePopUpForVMIcon(label, infoString, vmInfo);
						
						fD = infoLabel.getFont().getFontData();
						fD[0].setHeight(10);
						infoLabel.setFont(new Font(infoLabel.getDisplay(), fD[0]));						
						infoLabel.setText(infoString);
						infoLabel.pack();
						infoLabel.setLocation(x, y + icon_height - 26);
						
						vmLabelLists.add(label);
						vmLabelInfoLists.add(infoLabel);
					}
				}
			});
		}
	}
}