package edu.columbia.cs.irt.sipcloud.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;


import edu.columbia.cs.irt.sipcloud.Constants;
import edu.columbia.cs.irt.sipcloud.LoadScalingManager;
import org.eclipse.swt.widgets.TabFolder;

/**
 * MainWindow class of the visualization client
 * 
 * @author Jong-yul Kim, Jin Hyung Park [jp2105]
 * 
 */
public class MainWindow {

	protected Shell shlLoadScalingManager;
	
	private LoadScalingManager lsm;
	private CanvasUpdater canvasUpdater;
	private Thread canvasUpdaterThread;
	private TabFolder tabFolder;

	/**
	 * Open the main window.
	 */
	public void open() {
		Display display = Display.getDefault();
		createContents();
		shlLoadScalingManager.open();
		shlLoadScalingManager.layout();
		
		/*
		 * In canvasUpdaer thread, all VM's status will be updated.
		 */
		canvasUpdaterThread = new Thread(canvasUpdater);
		canvasUpdaterThread.start();
		
		while (!shlLoadScalingManager.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
	}

	/**
	 * Create contents of the window.
	 */
	protected void createContents() {
		shlLoadScalingManager = new Shell();
		shlLoadScalingManager.setSize(772, 660);
		shlLoadScalingManager.setText("Load Scaling Manager");
		shlLoadScalingManager.setLayout(null);
		
		Label label_1 = new Label(shlLoadScalingManager, SWT.SEPARATOR | SWT.HORIZONTAL);
		label_1.setBounds(0, 582, 771, 2);
		
		Button btnLaunchVm = new Button(shlLoadScalingManager, SWT.NONE);
		btnLaunchVm.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				final VmListDialog dialog = new VmListDialog(shlLoadScalingManager, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
				Listener listener = new Listener() {
					public void handleEvent (Event event) {
						dialog.shell.close();
						UIWaitingThread thread = new UIWaitingThread("StartInstance");
						thread.setStringArgument(dialog.comboSelection);
						thread.setBooleanArrayArgument(dialog.selectedVm);
						thread.setJobKind(Constants.UIThreadJob_LB_StartInstance);
						thread.start();
					}
				};
				dialog.setListener(listener);
				dialog.open();
			}
		});
		btnLaunchVm.setBounds(10, 590, 140, 40);
		btnLaunchVm.setText("Launch VM");
		
		Button btnDeleteVm = new Button(shlLoadScalingManager, SWT.NONE);
		btnDeleteVm.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				final StopInstanceDialog dialog = new StopInstanceDialog(shlLoadScalingManager, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
				Listener listener = new Listener() {
					public void handleEvent (Event event) {
						dialog.shell.close();
						UIWaitingThread thread = new UIWaitingThread("StopInstance");
						thread.setStringArgument(dialog.comboSelection);
						thread.setJobKind(Constants.UIThreadJob_LB_StopInstance);
						thread.start();
					}
				};
				dialog.setListener(listener);
				dialog.open();
			}
		});
		btnDeleteVm.setBounds(156, 590, 140, 40);
		btnDeleteVm.setText("Stop VM");
		
		Button btnStartCluster = new Button(shlLoadScalingManager, SWT.NONE);
		btnStartCluster.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				final RegionChooserDialog dialog = new RegionChooserDialog(shlLoadScalingManager, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
				Listener listener = new Listener() {
					public void handleEvent (Event event) {
						dialog.shell.close();
						UIWaitingThread thread = new UIWaitingThread("StartNewCluster");
						thread.setStringArgument(dialog.comboSelection);
						thread.setJobKind(Constants.UIThreadJob_LB_StartCluster);
						thread.start();
					}
				};
				dialog.setListener(listener);
				dialog.open();
			}
		});
		btnStartCluster.setBounds(300, 590, 140, 40);
		btnStartCluster.setText("Start Cluster");
		
		Button btnStopCluster = new Button(shlLoadScalingManager, SWT.NONE);
		btnStopCluster.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				final RegionChooserDialog dialog = new RegionChooserDialog(shlLoadScalingManager, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
				Listener listener = new Listener() {
					public void handleEvent (Event event) {
						dialog.shell.close();
						UIWaitingThread thread = new UIWaitingThread("StopCluster");
						thread.setStringArgument(dialog.comboSelection);
						thread.setJobKind(Constants.UIThreadJob_LB_StopCluster);
						thread.start();
					}
				};
				dialog.setListener(listener);
				dialog.open();			
			}
		});
		btnStopCluster.setBounds(446, 590, 140, 40);
		btnStopCluster.setText("Stop Cluster");
		
		tabFolder = new TabFolder(shlLoadScalingManager, SWT.NONE);
		tabFolder.setBounds(0, 0, 771, 576);
		
		canvasUpdater = new CanvasUpdater(shlLoadScalingManager, tabFolder);
	}
	
	/**
	 * LoadScalingManager Getter
	 * 
	 * @return lsm LoadScalingManager Object
	 */
	public LoadScalingManager getLsm() {
		return lsm;
	}

	/**
	 * LoadScalingManager Setter
	 * 
	 * @param lsm LoadScalingManager Object
	 */
	public void setLsm(LoadScalingManager lsm) {
		this.lsm = lsm;
	}
}