package edu.columbia.cs.irt.sipcloud.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

import edu.columbia.cs.irt.sipcloud.Constants;
import edu.columbia.cs.irt.sipcloud.LoadScalingManager;

/**
 * 
 * Right-click PopUp menu Controller
 * 
 * @author Jin Hyung Park [jp2105]
 *
 */
public class MenuController {
	private int x;
	private int y;
	
	/**
	 * This function creates the right-click pop-up menu
	 * for the given VM icon.
	 *  
	 * @param label This label will be 1st menu item to show what this menu is.
	 * @param menuDescription This string will be 2nd menu item to describe what this menu does. 
	 * @param vmInfo This string includes VM's information.
	 */
	public void makePopUpForVMIcon(final Label label, String menuDescription, String []vmInfo) {
		final Menu menu_1 = new Menu(label);
		label.setMenu(menu_1);
		menu_1.setData(Boolean.parseBoolean("false"));
		
		MenuItem mntmNewItemInfo_1 = new MenuItem(menu_1, SWT.NONE);
		mntmNewItemInfo_1.setText(menuDescription);
		mntmNewItemInfo_1.setEnabled(false);
		
		/*
		 * VM Re-configuration Menu 
		 */
		final MenuItem mntmNewItem0 = new MenuItem(menu_1, SWT.NONE);
		mntmNewItem0.setText("Re-config " + vmInfo[0]);
		mntmNewItem0.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				String []menuStr = mntmNewItem0.getText().split(" ");
				UIWaitingThread thread = new UIWaitingThread("StartNewCluster");
				thread.setStringArgument(menuStr[1]);
				thread.setJobKind(Constants.UIThreadJob_LB_ReConfigInstance);
				thread.start();
				menu_1.setData(Boolean.parseBoolean("false"));
			}
		});
		
		/*
		 * VM Stop Menu 
		 */
		final MenuItem mntmNewItem = new MenuItem(menu_1, SWT.NONE);
		mntmNewItem.setText("Stop " + vmInfo[0]);
		mntmNewItem.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				String []menuStr = mntmNewItem.getText().split(" ");
				UIWaitingThread thread = new UIWaitingThread("StartNewCluster");
				thread.setStringArgument(menuStr[1]);
				thread.setJobKind(Constants.UIThreadJob_LB_StartCluster);
				thread.start();
				menu_1.setData(Boolean.parseBoolean("false"));
			}
		});
		
		/*
		 * VM Terminate Menu 
		 */
		final MenuItem mntmNewItem_1 = new MenuItem(menu_1, SWT.NONE);
		mntmNewItem_1.setText("Terminate " + vmInfo[0]);
		mntmNewItem_1.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				String []menuStr = mntmNewItem_1.getText().split(" ");
				System.err.println("Called Terminate VM popup menu - " + menuStr[1]);
				LoadScalingManager.terminateInstanceUsingID(menuStr[1]);
				menu_1.setData(Boolean.parseBoolean("false"));
			}
		});
				
		/* 
		 * VM Status Menu 
		 */
		final MenuItem mntmNewItem_2 = new MenuItem(menu_1, SWT.NONE);
		mntmNewItem_2.setText("View Status");
		mntmNewItem_2.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				String []menuStr = mntmNewItem_1.getText().split(" ");
				System.err.println("Now it'll show up VM info - " + menuStr[1]);
				
				menu_1.setData(Boolean.parseBoolean("false"));
				
				final VMStatusInfoDialog dialog = new VMStatusInfoDialog(label.getShell(), SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
				dialog.setLocation(x, y);
				dialog.setInstanceID(menuStr[1]);
				
				Listener listener = new Listener() {
					public void handleEvent (Event event) {
						dialog.shell.close();
					}
				};
				dialog.setListener(listener);
				dialog.open();
			}
		});
		
		/*
		 * Moush Right click handling
		 */
		label.addMouseListener(new MouseListener() {
			@Override
			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub
				
			}
			@Override
			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub
			}
			@Override
			public void mouseUp(MouseEvent e) {
				// TODO Auto-generated method stub
				boolean isMenuUp = (Boolean) menu_1.getData();
				if (isMenuUp) {
					menu_1.setVisible(false);
					menu_1.setData(Boolean.parseBoolean("false"));
				} else {
					menu_1.setVisible(true);
					menu_1.setData(Boolean.parseBoolean("true"));
					x = e.x;
					y = e.y;
				}
			}
		});
	}
}
