package edu.columbia.cs.irt.sipcloud.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import edu.columbia.cs.irt.sipcloud.Constants;
import org.eclipse.swt.widgets.Button;

/**
 * RegionChooserDialog 
 * 
 * allows users to choose the Amazon EC2 region.
 *  
 * @author Jin Hyung Park [jp2105]
 *
 */
public class RegionChooserDialog extends Dialog {

	protected Object result;
	protected Shell shell;
	protected Listener mouseListener;
	protected String comboSelection = null;


	/**
	 * Create the dialog.
	 * @param parent Shell Parent
	 * @param style SWT Style
	 */
	public RegionChooserDialog(Shell parent, int style) {
		super(parent, style);
		setText("Choose region");
	}

	public void setListener(Listener mouseListener) {
		this.mouseListener = mouseListener;
	}
	
	/**
	 * Open the dialog.
	 * @return The region chooser dialog will be returned.
	 */
	public Object open() {
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() {
		shell = new Shell(getParent(), getStyle());
		shell.setSize(197, 114);
		shell.setText("Choose region");
		
		Combo combo = new Combo(shell, SWT.READ_ONLY);
		combo.setItems(new String[]{Constants.ec2UsEast, Constants.ec2UsWest});
		combo.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				comboSelection = ((Combo)e.widget).getText();
				System.out.println(comboSelection);
			}
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});
		combo.setBounds(10, 10, 171, 23);
		
		Button btnOK = new Button(shell, SWT.NONE);
		btnOK.addListener(SWT.MouseUp, mouseListener);
		btnOK.setBounds(10, 51, 75, 25);
		btnOK.setText("OK");
		
		Button btnCancel = new Button(shell, SWT.NONE);
		btnCancel.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				shell.close();
			}
		});
		btnCancel.setBounds(106, 51, 75, 25);
		btnCancel.setText("Cancel");

	}
}
