package edu.columbia.cs.irt.sipcloud.gui;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Label;

import edu.columbia.cs.irt.sipcloud.LoadScalingManager;

/**
 * StopInstanceDialog 
 * 
 * displays the list of VM instances so that users can stop the application running on VM instance.
 *  
 * @author Jin Hyung Park [jp2105]
 *
 */
public class StopInstanceDialog extends Dialog {

	protected Object result;
	protected Shell shell;
	protected Listener mouseListener;
	protected String comboSelection = null;

	/**
	 * Create the dialog.
	 * @param parent Shell Parent
	 * @param style SWT Style
	 */
	public StopInstanceDialog(Shell parent, int style) {
		super(parent, style);
		setText("Stop Instance");
	}
	
	public void setListener(Listener mouseListener) {
		this.mouseListener = mouseListener;
	}

	/**
	 * Open the dialog.
	 * @return The Stop Instance dialog will be returned.
	 */
	public Object open() {
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() {
		shell = new Shell(getParent(), getStyle());
		shell.setSize(320, 180);
		shell.setText(getText());
		
		Combo combo = new Combo(shell, SWT.READ_ONLY);
		String [] listOfInstances = LoadScalingManager.makeListOfInstances();
		combo.setItems(listOfInstances);
		combo.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				comboSelection = ((Combo)e.widget).getText();
				System.out.println(comboSelection);
			}
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});
		combo.setBounds(24, 49, 277, 23);
		
		Button btnOK = new Button(shell, SWT.NONE);
		btnOK.addListener(SWT.MouseUp, mouseListener);
		btnOK.setBounds(62, 107, 75, 25);
		btnOK.setText("OK");
		
		Button btnCancel = new Button(shell, SWT.NONE);
		btnCancel.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				shell.close();
			}
		});
		btnCancel.setBounds(177, 107, 75, 25);
		btnCancel.setText("Cancel");
		
		Label lblChooseAnInstance = new Label(shell, SWT.NONE);
		lblChooseAnInstance.setBounds(24, 10, 224, 32);
		lblChooseAnInstance.setText("Choose a component to stop.");

	}
}
