package edu.columbia.cs.irt.sipcloud.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;

/**
 * TabItem Abstract class represents each Amazon EC2 region.
 * 
 * Also, this includes 3 area canvases for each Amazon EC2 region.
 * By adding this class, LoadScalingManager can easily add another region. 
 * 
 * @author Jin Hyung Park [jp2105]
 * 
 */
public class TabbarDataCenter {
	// TabFolder's name : Actually, this name represents the Amazon EC2's region.
	private String tabName;
	
	public TabItem tabItem_1;
	
	// canvas_1 : LoadBalancer, Proxy, Database will be here.
	public Canvas canvas_1;
	// canvas_2 : Media Relay, Conference will be here.
	public Canvas canvas_2;
	// canvas_3 : Preliminary VMs will be here.
	public Canvas canvas_3;

	/**
	 * TabbarDataCenter Constructor
	 * 
	 * @param tabFolder TabFolder class - Our TabItem will be added to this tabFolder.
	 * @param tabName TabFolder's Tab Name. In our case, this will be Amazon EC2 Region name.
	 */
	public TabbarDataCenter(TabFolder tabFolder, String tabName) {
		this.tabName = tabName;
		
		tabItem_1 = new TabItem(tabFolder, SWT.NONE);
		tabItem_1.setText(tabName);
		
		final Canvas canvas = new Canvas(tabFolder, SWT.NONE);
		tabItem_1.setControl(canvas);
		
		ScrolledComposite scrolledComposite = new ScrolledComposite(canvas, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite.setBounds(5, 10, 500, 250);
		scrolledComposite.setExpandHorizontal(false);
		scrolledComposite.setExpandVertical(false);
		
		/* canvas_1 is for Load Balancer, Proxy, and Database */
		canvas_1 = new Canvas(scrolledComposite, SWT.NONE);
		canvas_1.setSize(500, 1000);
		scrolledComposite.setContent(canvas_1);
		scrolledComposite.setMinSize(canvas_1.getSize());
		
		ScrolledComposite scrolledComposite_1 = new ScrolledComposite(canvas, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite_1.setBounds(5, 270, 500, 250);
		scrolledComposite_1.setExpandHorizontal(false);
		scrolledComposite_1.setExpandVertical(false);
		
		/* canvase_2 is for Media Relay and Conference */
		canvas_2 = new Canvas(scrolledComposite_1, SWT.NONE);
		scrolledComposite_1.setContent(canvas_2);
		scrolledComposite_1.setMinSize(canvas_2.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		canvas_2.setSize(500, 1000);
		
		ScrolledComposite scrolledComposite_2 = new ScrolledComposite(canvas, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		scrolledComposite_2.setBounds(515, 10, 230, 510);
		scrolledComposite_2.setExpandHorizontal(false);
		scrolledComposite_2.setExpandVertical(false);
		
		/* canvans_3 is for preliminary VMs */
		canvas_3 = new Canvas(scrolledComposite_2, SWT.NONE);
		scrolledComposite_2.setContent(canvas_3);
		scrolledComposite_2.setMinSize(canvas_3.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		canvas_3.setSize(230, 1000);
	}
	
	/**
	 * Get this tab name.
	 * 
	 * @return Tab Name(Amazon EC2 Region name)
	 */
	public String getTabName() {
		return this.tabName;
	}
	
	/**
	 * Set this tab name.
	 * 
	 * @param tabName Name which will be displayed on TabFolder.
	 */
	public void setTabName(String tabName) {
		this.tabName = tabName;
	}
}