package edu.columbia.cs.irt.sipcloud.gui;

import edu.columbia.cs.irt.sipcloud.Constants;
import edu.columbia.cs.irt.sipcloud.LoadScalingManager;

/**
 * UIWaitingThread 
 * 
 * makes UI wait until finishing LoadScalingManager's job
 *  
 * @author Jin Hyung Park [jp2105]
 *
 */
public class UIWaitingThread extends Thread {
	private int jobKind;
	private String stringArgument;
	private boolean []boolArrayArgument;
	
	/**
	 * Inherits Thread's class constructor
	 * 
	 * @param str Thread Name
	 */
	public UIWaitingThread(String str) {
		super(str);
	}
	
	/**
	 * String Type Argument for calling LoadScalingManger's function
	 * 
	 * @param arg String Argument
	 */
	public void setStringArgument(String arg) {
		this.stringArgument = arg;
	}
	
	/**
	 * Get the current string argument
	 * 
	 * @return string argument
	 */
	public String getStringArgument() {
		return stringArgument;
	}
	
	/**
	 * Set the boolean array argument
	 * 
	 * @param arg Boolean array
	 */
	public void setBooleanArrayArgument(boolean [] arg) {
		this.boolArrayArgument = arg;
	}
	
	/**
	 * Get the current boolean array argument
	 * 
	 * @return Boolean array
	 */
	public boolean [] getBooleanArrayArgument() {
		return this.boolArrayArgument;
	}
	
	/**
	 * Set the kind of job
	 * 
	 * @see Constants Class
	 * @param jobKind Job Kind in Constants Class
	 */
	public void setJobKind(int jobKind) {
		this.jobKind = jobKind;
	}
	
	/**
	 * Get the current job kind
	 * 
	 * @return Job Kind in Constants Class
	 */
	public int getJobKind() {
		return jobKind;
	}
	
	/**
	 * Inherits Thread(Super Class)'s run function
	 * 
	 * In MainWindow.java, to lunch or stop VM,
	 * we call LoadSclingManager's function; however,
	 * this function uses blocking I/O, so, during that time,
	 * GUI is blocked. So, we prevent it by calling those functions
	 * in thread(this class). 
	 */
	public void run() {
		if (jobKind == Constants.UIThreadJob_LB_StartCluster)
		{
			LoadScalingManager.startNewCluster(stringArgument);
		}
		else if (jobKind == Constants.UIThreadJob_LB_StopCluster)
		{
			LoadScalingManager.stopCluster(stringArgument);
		}
		else if (jobKind == Constants.UIThreadJob_LB_StartInstance)
		{
			LoadScalingManager.addNewVms(stringArgument, boolArrayArgument);
		}
		else if (jobKind == Constants.UIThreadJob_LB_StopInstance)
		{			
			LoadScalingManager.stopInstanceUsingID(stringArgument);
		}
		else if (jobKind == Constants.UIThreadJob_LB_ReConfigInstance)
		{
			LoadScalingManager.reconfigInstanceUsingID(stringArgument);
		}
	}
}