package edu.columbia.cs.irt.sipcloud.gui;

import edu.columbia.cs.irt.sipcloud.LoadScalingManager;
import com.amazonaws.services.cloudwatch.model.Datapoint;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;

import java.awt.Color;
import java.text.SimpleDateFormat;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.Range;
import org.jfree.data.time.Minute;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.time.TimeSeriesDataItem;
import org.jfree.data.xy.XYDataset;
import org.jfree.experimental.chart.swt.ChartComposite;
import org.jfree.ui.RectangleInsets;

/**
 * VMStatusInfoDialog 
 * 
 * shows the statistics information graph. 
 *  
 * @author Jin Hyung Park [jp2105]
 *
 */
public class VMStatusInfoDialog extends Dialog {

	private static final Object result = null;
	protected Shell shell;
	//private Listener mouseListener;
	private int x;
	private int y;
	private String instanceID;
	private ArrayList<Datapoint> res1;
	private ArrayList<Datapoint> res2;
	private ArrayList<Datapoint> res3;
	
	public String instancePubIP;
	public String instancePrvIP;
	public String instanceType;
	public String instanceLoc;
	
	private JFreeChart chart1;
	private TimeSeriesCollection dataSet1;
	private JFreeChart chart2;
	private TimeSeriesCollection dataSet2;
	private JFreeChart chart3;
	private TimeSeriesCollection dataSet3;
	
	/**
     * Creates a chart.
     * 
     * @param dataset  a dataset.
     * 
     * @return A chart.
     */
    private static JFreeChart createChart(String timeName, String yStr, XYDataset dataset) {

        JFreeChart chart = ChartFactory.createTimeSeriesChart(
            timeName,  // title
            "Date",             // x-axis label
            yStr,   // y-axis label
            dataset,            // data
            true,               // create legend?
            true,               // generate tooltips?
            false               // generate URLs?
        );

        chart.setBackgroundPaint(Color.white);

        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setBackgroundPaint(Color.lightGray);
        plot.setDomainGridlinePaint(Color.white);
        plot.setRangeGridlinePaint(Color.white);
        plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
        plot.setDomainCrosshairVisible(true);
        plot.setRangeCrosshairVisible(true);
        
        XYItemRenderer r = plot.getRenderer();
        if (r instanceof XYLineAndShapeRenderer) {
            XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
            renderer.setBaseShapesVisible(true);
            renderer.setBaseShapesFilled(true);
        }
        
        DateAxis axis = (DateAxis) plot.getDomainAxis();
        axis.setDateFormatOverride(new SimpleDateFormat("HH:mm"));

        if (timeName.equalsIgnoreCase("CPU Utilization")) {
        	ValueAxis yaxis = plot.getRangeAxis();
        	yaxis.setRange(new Range(0, 100));
        } else {
        	ValueAxis yaxis = plot.getRangeAxis();
        	yaxis.setRange(new Range(0, 1024));
        	
        	TimeSeriesCollection timeDataset = (TimeSeriesCollection) dataset;
        	TimeSeries s1 = timeDataset.getSeries(0);
        	
        	List <TimeSeriesDataItem> list = s1.getItems();
        	double max = 0;
        	for (int i = 0; i < list.size(); ++i) {
        		TimeSeriesDataItem d1 = list.get(i);
        		if (d1.getValue().doubleValue() > max)
        			max = d1.getValue().doubleValue();
        	}
        	
        	yaxis.setRange(new Range(0, max));

        }
        
        return chart;

    }
    
    /**
     * Creates a dataset, consisting of two series of monthly data.
     *
     * @return The dataset.
     */
    private static XYDataset createDataset(String timeName, ArrayList <Datapoint> res) {

        TimeSeries s1 = new TimeSeries(timeName);
        
        s1.setDescription("Description");
        s1.setDomainDescription("Domain Desc");
        s1.setRangeDescription("RangeDesc");
        
        for (int i = 0; i < res.size(); ++i) {
        	Datapoint pt = res.get(i);
        	
        	Minute min = new Minute(pt.getTimestamp());
        	
        	if (timeName.equalsIgnoreCase("CPU Utilization"))
        		s1.add(min, pt.getAverage());
        	else {
        		if (pt.getAverage() < 2)
        			s1.add(min, 0);
        		else
        			s1.add(min, pt.getAverage()/1024);
        	}
        }
        
        if (res.size() == 0) {
        	Minute min = new Minute();
        	s1.add(min, 0.0);
        } else {
        	System.out.println("==> " + res);
        }

        TimeSeriesCollection dataset = new TimeSeriesCollection();
        dataset.addSeries(s1);
        
        return dataset;
    }
    
    private void updateDataset(String timeName, ArrayList <Datapoint> res) {

        TimeSeries s1 = new TimeSeries(timeName);
        
        s1.setDescription("Description");
        s1.setDomainDescription("Domain Desc");
        s1.setRangeDescription("RangeDesc");
        
        for (int i = 0; i < res.size(); ++i) {
        	Datapoint pt = res.get(i);
        	
        	Minute min = new Minute(pt.getTimestamp());
        	
        	if (timeName.equalsIgnoreCase("CPU Utilization"))
        		s1.add(min, pt.getAverage());
        	else {
        		if (pt.getAverage() < 2)
        			s1.add(min, 0);
        		else
        			s1.add(min, pt.getAverage()/1024);
        	}
        }
        
        if (res.size() == 0) {
        	Minute min = new Minute();
        	s1.add(min, 0.0);
        } else {
        	System.out.println("==> " + res);
        }

        if (timeName.equalsIgnoreCase("Network In")) {
        	dataSet1.removeAllSeries();
	        dataSet1.addSeries(s1);
        }
        else if (timeName.equalsIgnoreCase("Network Out")) {
        	dataSet2.removeAllSeries();
	        dataSet2.addSeries(s1);
        }
        else if (timeName.equalsIgnoreCase("CPU Utilization")) {
        	dataSet3.removeAllSeries();
	        dataSet3.addSeries(s1);
        }
    }
    

	public VMStatusInfoDialog(Shell parent, int style) {
		super(parent, style);
	}

	public void setListener(Listener mouseListener) {
		//this.mouseListener = mouseListener;
	}
	
	public void setInstanceID(String instanceID) {
		this.instanceID = instanceID;
		String vmLists[] = LoadScalingManager.makeListOfInstances();
		for (int i = 0; i < vmLists.length; ++i) {
			String vmInfo[] = vmLists[i].split(" ");
			if (vmInfo[0].equalsIgnoreCase(instanceID)) {
				this.instanceType = vmInfo[1];
				this.instancePubIP = vmInfo[2];
				this.instancePrvIP = vmInfo[3];
				this.instanceLoc = vmInfo[4];
			}
		}
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() {
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		Date date = new Date();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
				Date newDate = new Date();
				if ((newDate.getTime() - date.getTime()) > 60000) { /* 60000 is 1 minute */
					date = newDate;
					updateContents();
				}
			}
		}
		return result;
	}
	
	public void getVMStatistic() {
		ArrayList<List<Datapoint>> resultList = LoadScalingManager.getVMStatistic(instanceID);
		
		res1 = (ArrayList<Datapoint>) resultList.get(0); /* Network In */
		res2 = (ArrayList<Datapoint>) resultList.get(1); /* Network Out */
		res3 = (ArrayList<Datapoint>) resultList.get(2); /* CPU Utilization */
	}
	
	/**
	 * Update contents of the dialog.
	 */
	private void updateContents() {
		this.getVMStatistic();
		
		this.updateDataset("Network In", res1);
		this.updateDataset("Network Out", res2);
		this.updateDataset("CPU Utilization", res3);
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() {
		shell = new Shell(getParent(), getStyle());
		shell.setSize(620, 640);
		//shell.setLayout(new FillLayout());
		shell.setLocation(this.x, this.y);
		
		this.getVMStatistic();
		
		GridLayout gridLayout = new GridLayout();
	    gridLayout.numColumns = 2;
	    gridLayout.makeColumnsEqualWidth = true;
		
		shell.setLayout(gridLayout);

	    GridData gridData = new GridData();
	    gridData.widthHint = 300;
	    gridData.heightHint = 300;
	    
	    /* #1 Graph */
	    dataSet1 = (TimeSeriesCollection) createDataset("Network In", this.res1);
		chart1 = createChart("Network In", "KBytes", dataSet1);
		chart1.setTitle("Network In");
		ChartComposite frame1 = new ChartComposite(shell, SWT.PUSH, chart1, true);
		frame1.setLayoutData(gridData);
        frame1.setDisplayToolTips(true);
        frame1.setHorizontalAxisTrace(false);
        frame1.setVerticalAxisTrace(false);
        
        /* #2 Graph */
        dataSet2 = (TimeSeriesCollection) createDataset("Network Out", this.res2);
		chart2 = createChart("Network Out", "KBytes", dataSet2);
		chart2.setTitle("Network Out");
		ChartComposite frame2 = new ChartComposite(shell, SWT.PUSH, chart2, true);
		frame2.setLayoutData(gridData);
        frame2.setDisplayToolTips(true);
        frame2.setHorizontalAxisTrace(false);
        frame2.setVerticalAxisTrace(false);
        
        /* #3 Graph */
        dataSet3 = (TimeSeriesCollection) createDataset("CPU Utilization", this.res3);
		chart3 = createChart("CPU Utilization", "Percent", dataSet3);
		chart3.setTitle("CPU Utilization");
		ChartComposite frame3 = new ChartComposite(shell, SWT.PUSH, chart3, true);
		frame3.setLayoutData(gridData);
        frame3.setDisplayToolTips(true);
        frame3.setHorizontalAxisTrace(false);
        frame3.setVerticalAxisTrace(false);
        

        final Composite composite = new Composite(shell, SWT.PUSH);
        composite.setSize(300, 300);
        composite.setLayoutData(gridData);
        
		
		Label title = new Label(composite, SWT.NONE);
		title.setLayoutData(gridData);
		title.setBounds(50, 50, 250, 25);
		title.setText("Instance " + this.instanceID + " Statistics");
		
		Label title2 = new Label(composite, SWT.NONE);
		title2.setLayoutData(gridData);
		title2.setBounds(50, 75, 250, 25);
		title2.setText("Public IP " + this.instancePubIP);
		
		Label title3 = new Label(composite, SWT.NONE);
		title3.setLayoutData(gridData);
		title3.setBounds(50, 100, 250, 25);
		title3.setText("Private IP " + this.instancePrvIP);
		
		Label title5 = new Label(composite, SWT.NONE);
		title5.setLayoutData(gridData);
		title5.setBounds(50, 125, 250, 25);
		title5.setText("Instance Location");
		
		Label title4 = new Label(composite, SWT.NONE);
		title4.setLayoutData(gridData);
		title4.setBounds(50, 150, 250, 25);
		title4.setText("    " + this.instanceLoc);
		
		Button btnCancel = new Button(composite, SWT.NONE);
		btnCancel.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				shell.close();
			}
		});
		btnCancel.setBounds(200, 250, 75, 25);
		btnCancel.setText("Close");
		
	}

	public void setLocation(int x, int y) {
		this.x = x;
		this.y = y;
	}
}