package edu.columbia.cs.irt.sipcloud.gui;

import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;

import edu.columbia.cs.irt.sipcloud.Constants;
import org.eclipse.swt.widgets.Combo;

/**
 * VmListDialog 
 * 
 * displays the list of the type of VM instances so that users can choose the VM type. 
 *  
 * @author Jin Hyung Park [jp2105]
 *
 */
public class VmListDialog extends Dialog {

	protected Object result;
	protected Shell shell;
	protected Listener mouseListener;
	protected boolean[] selectedVm;
	private int NUMOFSELECTION;	
	protected String comboSelection = null;

	/**
	 * Create the dialog.
	 * @param parent
	 * @param style
	 */
	public VmListDialog(Shell parent, int style) {
		super(parent, style);
		setText("SWT Dialog");
		
		NUMOFSELECTION = Constants.VmTypes.values().length;
		selectedVm = new boolean[NUMOFSELECTION];
		for (int i = 0; i < NUMOFSELECTION; i++) {
			selectedVm[i] = false;
		}
	}
	
	public void setListener(Listener mouseListener) {
		this.mouseListener = mouseListener;
	}

	/**
	 * Open the dialog.
	 * @return the result
	 */
	public Object open() {
		createContents();
		shell.open();
		shell.layout();
		Display display = getParent().getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}
		return result;
	}

	/**
	 * Create contents of the dialog.
	 */
	private void createContents() {
		shell = new Shell(getParent(), getStyle());
		shell.setSize(230, 358);
		
		Label lblChooseRegion = new Label(shell, SWT.NONE);
		lblChooseRegion.setText("1. Choose region");
		lblChooseRegion.setBounds(10, 10, 204, 25);
		
		Combo combo = new Combo(shell, SWT.READ_ONLY);
		combo.setItems(new String[]{Constants.ec2UsEast, Constants.ec2UsWest});
		combo.addSelectionListener(new SelectionListener() {
			public void widgetSelected(SelectionEvent e) {
				comboSelection = ((Combo)e.widget).getText();
				System.out.println(comboSelection);
			}
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});
		combo.setBounds(31, 41, 158, 23);
		
		
		Label lblSelectVmsTo = new Label(shell, SWT.NONE);
		lblSelectVmsTo.setBounds(10, 87, 204, 25);
		lblSelectVmsTo.setText("2. Select VMs to launch");
		
		final Button btnLoadBalancer = new Button(shell, SWT.CHECK);
		btnLoadBalancer.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectedVm[Constants.VmTypes.LB.ordinal()] = btnLoadBalancer.getSelection();
			}
		});
		btnLoadBalancer.setBounds(50, 105, 120, 30);
		btnLoadBalancer.setText("Load Balancer");
		
		final Button btnProxy = new Button(shell, SWT.CHECK);
		btnProxy.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectedVm[Constants.VmTypes.PROXY.ordinal()] = btnProxy.getSelection();
			}
		});
		btnProxy.setBounds(50, 141, 120, 30);
		btnProxy.setText("Proxy");
		
		final Button btnCassandra = new Button(shell, SWT.CHECK);
		btnCassandra.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectedVm[Constants.VmTypes.DB.ordinal()] = btnCassandra.getSelection();
			}
		});
		btnCassandra.setBounds(50, 177, 120, 30);
		btnCassandra.setText("Database");
		
		final Button btnMediaRelay = new Button(shell, SWT.CHECK);
		btnMediaRelay.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectedVm[Constants.VmTypes.RELAY.ordinal()] = btnMediaRelay.getSelection();
			}
		});
		btnMediaRelay.setBounds(50, 213, 120, 30);
		btnMediaRelay.setText("Media relay");
		
		final Button btnConference = new Button(shell, SWT.CHECK);
		btnConference.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectedVm[Constants.VmTypes.CONFERENCE.ordinal()] = btnConference.getSelection();
			}
		});
		btnConference.setBounds(50, 249, 120, 30);
		btnConference.setText("Conference");

		Button btnLaunch = new Button(shell, SWT.NONE);
		btnLaunch.addListener(SWT.MouseUp, mouseListener);
		btnLaunch.setBounds(10, 295, 75, 25);
		btnLaunch.setText("Launch");
		
		Button btnCancel = new Button(shell, SWT.NONE);
		btnCancel.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseUp(MouseEvent e) {
				shell.close();
			}
		});
		btnCancel.setBounds(139, 295, 75, 25);
		btnCancel.setText("Cancel");
		
		
	}
}
